#decorators
#@normalized_function(xmin,xmax)
#def function_i_have_no_idea_how_to_normalize(x,y,z)
#   return complicated_function(x,y,z)
#
from .functor import Normalized, Extended
class normalized:
    """
    Normalized decorator

    **Arguments**
        - **bound** normalized bound
        - **nint** option number of integral pieces. Default 1000.

    .. seealso::

        :class:`probfit.functor.Normalized`

    """

    def __init__(self, bound, nint=1000):
        self.bound  = bound
        self.nint = nint

    def __call__(self,f):
        return Normalized(f, self.bound, self.nint)


class extended:
    """
    Extended decorator

    **Arguments**
        - **extname** extended parameter name. Default 'N'

    .. seealso::

        :class:`probfit.functor.Extended`

    """
    def __init__(self, extname='N'):
        self.extname = extname

    def __call__(self,f):
        return Extended(f, extname=self.extname)



import unittest
from probfit.pdf import *
from probfit.util import *
from probfit.costfunc import *
from math import log
import numpy as np
from numpy.random import randn
from probfit._libstat import xlogyx, wlogyx, csum, integrate1d, _vector_apply
from probfit.functor import Normalized, AddPdf, construct_arg, fast_tuple_equal
from probfit.funcutil import merge_func_code

from nose.tools import *

def iterable_equal(x, y):
    assert_equal(list(x), list(y))

#cpdef double doublegaussian(double x, double mean, double sigmal, double sigmar)
def test_doublegaussian():
    assert_equal(tuple(describe(doublegaussian)), ('x','mean','sigmal','sigmar'))
    assert_almost_equal(doublegaussian(0.,0.,1.,2.),1.)
    assert_almost_equal(doublegaussian(-1.,0.,1.,2.), 0.6065306597126334)
    assert_almost_equal(doublegaussian(1.,0.,1.,2.), 0.8824969025845955)

#cpdef double ugaussian(double x, double mean, double sigma)
def test_ugaussian():
    assert_equal(tuple(describe(ugaussian)), ('x','mean','sigma'))
    assert_almost_equal(ugaussian(0,0,1), 1.)
    assert_almost_equal(ugaussian(-1,0,1), 0.6065306597126334)
    assert_almost_equal(ugaussian(1,0,1), 0.6065306597126334)

#cpdef double gaussian(double x, double mean, double sigma)
def test_gaussian():
    assert_equal(tuple(describe(gaussian)), ('x','mean','sigma'))
    assert_almost_equal(gaussian(0,0,1), 0.3989422804014327)
    assert_almost_equal(gaussian(-1,0,1), 0.24197072451914337)
    assert_almost_equal(gaussian(1,0,1), 0.24197072451914337)

#cpdef double crystalball(double x,double alpha,double n,double mean,double sigma)
def test_crystalball():
    assert_equal(tuple(describe(crystalball)), ('x','alpha','n','mean','sigma'))
    assert_almost_equal(crystalball(10,1,2,10,2),1.)
    assert_almost_equal(crystalball(11,1,2,10,2),0.8824969025845955)
    assert_almost_equal(crystalball(12,1,2,10,2),0.6065306597126334)
    assert_almost_equal(crystalball(14,1,2,10,2),0.1353352832366127)
    assert_almost_equal(crystalball(6,1,2,10,2),0.26956918209450376)

#cpdef double argus(double x, double c, double chi, double p)
def test_argus():
    assert_equal(tuple(describe(argus)), ('x','c','chi','p'))
    assert_almost_equal(argus(6.,10,2,3),0.004373148605400128)
    assert_almost_equal(argus(10.,10,2,3),0.)
    assert_almost_equal(argus(8.,10,2,3),0.0018167930603254737)

#cpdef double cruijff(double x, double m0, double sigma_L, double sigma_R, double alpha_L, double alpha_R)
def test_cruijff():
    iterable_equal(tuple(describe(cruijff)),
        ('x', 'm0', 'sigma_L', 'sigma_R', 'alpha_L', 'alpha_R'))
    val = cruijff(0,0,1.,2.,1.,2.)
    assert_almost_equal(val,1.)
    vl = cruijff(0,1,1.,1.,2.,2.)
    vr = cruijff(2,1,1.,1.,2.,2.)
    assert_almost_equal(vl,vr,msg='symmetric test')
    assert_almost_equal(vl,0.7788007830714)
    assert_almost_equal(vr,0.7788007830714)

#cpdef double linear(double x, double m, double c)
def test_linear():
    assert_equal(describe(linear),['x','m','c'])
    assert_almost_equal(linear(1,2,3),5)

#cpdef double poly2(double x, double a, double b, double c)
def test_poly2():
    assert_equal(describe(poly2),['x','a','b','c'])
    assert_almost_equal(poly2(2,3,4,5),25)

#cpdef double poly3(double x, double a, double b, double c, double d)
def test_poly2():
    assert_equal(describe(poly3),['x','a','b','c','d'])
    assert_almost_equal(poly3(2,3,4,5,6),56.)

#cpdef double novosibirsk(double x, double width, double peak, double tail)
def test_novosibirsk():
    assert_equal(describe(novosibirsk), ['x','width', 'peak', 'tail'])
    assert_almost_equal(novosibirsk(3,2,3,4),1.1253517471925912e-07)


def test_rtv_breitwigner():
    assert_equal(describe(rtv_breitwigner),['x','m','gamma'])
    assert_almost_equal(rtv_breitwigner(1,1,1.), 0.8194496535636714)
    assert_almost_equal(rtv_breitwigner(1,1,2.), 0.5595531041435416)
    assert_almost_equal(rtv_breitwigner(1,2,3.), 0.2585302502852219)


def test_cauchy():
    assert_equal(describe(cauchy),['x','m','gamma'])
    assert_almost_equal(cauchy(1,1,1.), 0.3183098861837907)
    assert_almost_equal(cauchy(1,1,2.), 0.15915494309189535)
    assert_almost_equal(cauchy(1,2,4.), 0.07489644380795074)


def test__vector_apply():
    def f(x,y): return x*x+y
    y = 10
    a = np.array([1.,2.,3.])
    expected = [f(x,y) for x in a]
    va = _vector_apply(f,a,tuple([y]))
    for i in range(len(a)):
        assert_almost_equal(va[i], expected[i])

def test_integrate1d():
    def f(x,y):return x*x+y
    def intf(x,y): return x*x*x/3.+y*x
    bound = (-2.,1.)
    y = 3.
    integral = integrate1d(f,bound,1000,tuple([y]))
    analytic = intf(bound[1],y)-intf(bound[0],y)
    assert_almost_equal(integral, 12.000004509013694)

def test_csum():
    x = np.array([1,2,3],dtype=np.double)
    s = csum(x)
    assert_almost_equal(s,6.)

def test_xlogyx():
    def bad(x,y): return x*log(y/x)
    assert_almost_equal(xlogyx(1.,1.),bad(1.,1.))
    assert_almost_equal(xlogyx(1.,2.),bad(1.,2.))
    assert_almost_equal(xlogyx(1.,3.),bad(1.,3.))
    assert_almost_equal(xlogyx(0.,1.),0.)

def test_wlogyx():
    def bad(w,y,x): return w*log(y/x)
    assert_almost_equal(wlogyx(1.,1.,1.),bad(1.,1.,1.))
    assert_almost_equal(wlogyx(1.,2.,3.),bad(1.,2.,3.))
    assert_almost_equal(wlogyx(1e-50,1e-20,1.),bad(1e-50,1e-20,1.))

def test_construct_arg():
    arg = (1,2,3,4,5,6)
    pos = np.array([0,2,4],dtype=np.int)
    carg = construct_arg(arg,pos)
    expected = (1,3,5)
    #print carg
    for i in range(len(carg)):
        assert_almost_equal(carg[i],expected[i])

def test_merge_func_code():
    def f(x,y,z): return x+y+z
    def g(x,a,b): return x+a+b
    def h(x,c,d): return x+c+d

    funccode, [pf,pg,ph] = merge_func_code(f,g,h)
    assert_equal(funccode.co_varnames,('x','y','z','a','b','c','d'))
    exp_pf = [0,1,2]
    for i in range(len(pf)): assert_almost_equal(pf[i],exp_pf[i])
    exp_pg = [0,3,4]
    for i in range(len(pf)): assert_almost_equal(pg[i],exp_pg[i])
    exp_ph = [0,5,6]
    for i in range(len(pf)): assert_almost_equal(ph[i],exp_ph[i])

    funccode, [pf,pg,ph] = merge_func_code(f,g,h,prefix=['f_','g_','h_'],skip_first=True)
    assert_equal(funccode.co_varnames,('x','f_y','f_z','g_a','g_b','h_c','h_d'))
    exp_pf = [0,1,2]
    for i in range(len(pf)): assert_almost_equal(pf[i],exp_pf[i])
    exp_pg = [0,3,4]
    for i in range(len(pf)): assert_almost_equal(pg[i],exp_pg[i])
    exp_ph = [0,5,6]
    for i in range(len(pf)): assert_almost_equal(ph[i],exp_ph[i])

def test_fast_tuple_equal():
    a = (1.,2.,3.)
    b = (1.,2.,3.)
    ret = fast_tuple_equal(a,b,0)
    assert(ret)

    a = (1.,4.,3.)
    b = (1.,2.,3.)
    ret = fast_tuple_equal(a,b,0)
    assert(not ret)

    a = (4.,3.)
    b = (1.,4.,3.)
    ret = fast_tuple_equal(a,b,1)
    assert(ret)

    a = (4.,5.)
    b = (1.,4.,3.)
    ret = fast_tuple_equal(a,b,1)
    assert(not ret)

    a = tuple([])
    b = tuple([])
    ret = fast_tuple_equal(a,b,0)
    assert(ret)



import unittest
from probfit import *
from probfit.pdf import  gaussian
from probfit.functor import Extended
import numpy as np
from numpy.random import randn, seed
from math import sqrt
from iminuit.iminuit_warnings import InitialParamWarning
import warnings

def assert_almost_equal(x,y,delta=1e-7):
    assert(y-delta < x < y+delta)

class TestOneshot(unittest.TestCase):

    def setUp(self):
        self.ndata = 20000
        warnings.simplefilter("ignore", InitialParamWarning)
        seed(0)
        self.data = randn(self.ndata)*2. + 5.
        self.wdown = np.empty(self.ndata)
        self.wdown.fill(0.1)

        self.ndata_small = 2000
        self.data_small = randn(self.ndata_small)*2. + 5.

    def test_binx2(self):
        egauss = Extended(gaussian)
        fit,m = fit_binx2(egauss, self.data, bins=100, bound=(1.,9.),
            quiet=True, mean=4., sigma=1., N=10000., print_level=0)
        assert_almost_equal(m.values['mean'], 5., delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'], 2., delta=3*m.errors['sigma'])

    def test_binlh(self):
        ngauss=Normalized(gaussian,(1.,9.))
        fit,m = fit_binlh(ngauss, self.data,bins=1000, bound=(1.,9.), quiet=True,
            mean=4., sigma=1.5, print_level=0)
        assert_almost_equal(m.values['mean'],5.,delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'],2.,delta=3*m.errors['sigma'])

    def test_extended_binlh(self):
        egauss = Extended(gaussian)
        fit,m = fit_binlh(egauss,self.data, bins=1000, bound=(1.,9.), quiet=True,
            mean=4., sigma=1., N=10000.,
            print_level=0, extended=True)
        assert_almost_equal(m.values['mean'],5.,delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'],2.,delta=3*m.errors['sigma'])
        assert_almost_equal(m.values['N'],20000,delta=3*m.errors['N'])

    def test_extended_binlh_ww(self):
        egauss = Extended(gaussian)
        fit,m = fit_binlh(egauss,self.data,bins=1000, bound=(1.,9.), quiet=True,
            mean=4., sigma=1.,N=1000., weights=self.wdown,
            print_level=-1, extended=True)
        assert_almost_equal(m.values['mean'],5.,delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'],2.,delta=3*m.errors['sigma'])
        assert_almost_equal(m.values['N'],2000,delta=3*m.errors['N'])

    def test_extended_binlh_ww_w2(self):
        egauss = Extended(gaussian)
        fit,m = fit_binlh(egauss,self.data,bins=1000, bound=(1.,9.), quiet=True,
            mean=4., sigma=1.,N=1000., weights=self.wdown,
            print_level=-1, extended=True)
        assert_almost_equal(m.values['mean'],5.,delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'],2.,delta=3*m.errors['sigma'])
        assert_almost_equal(m.values['N'],2000,delta=3*m.errors['N'])

        fit2,m2 = fit_binlh(egauss,self.data,bins=1000, bound=(1.,9.), quiet=True,
            mean=4., sigma=1.,N=1000., weights=self.wdown,
            print_level=-1, extended=True, use_w2=True)
        #assert_almost_equal(m2.values['mean'],5.,delta=3*m2.errors['mean'])
        assert_almost_equal(m2.values['sigma'],2.,delta=3*m2.errors['sigma'])
        assert_almost_equal(m2.values['N'], 2000., delta=3*m2.errors['N'])
        m.minos()
        m2.minos()
        #now error should scale correctly
        assert_almost_equal( m.errors['mean']/sqrt(10),
                                m2.errors['mean'],
                                delta = m.errors['mean']/sqrt(10)/100.)
        assert_almost_equal(m.errors['sigma']/sqrt(10),
                               m2.errors['sigma'],
                               delta = m.errors['sigma']/sqrt(10)/100.)
        assert_almost_equal(m.errors['N']/sqrt(10),
                               m2.errors['N'],
                               delta = m.errors['N']/sqrt(10)/100.)

    def test_uml(self):
        fit,m = fit_uml(gaussian, self.data, quiet=True,
                        mean=4.5, sigma=1.5, print_level=0)
        assert_almost_equal(m.values['mean'],5.,delta=3*m.errors['mean'])
        assert_almost_equal(m.values['sigma'],2.,delta=3*m.errors['sigma'])

if __name__ == '__main__':
    unittest.main()



from matplotlib.testing.decorators import image_comparison
from probfit.plotting import *
from probfit.pdf import gaussian, linear
from probfit.funcutil import rename
from probfit.functor import Extended, AddPdfNorm, AddPdf
from probfit.costfunc import UnbinnedLH, BinnedLH, BinnedChi2, Chi2Regression,\
                             SimultaneousFit
from matplotlib.testing.compare import compare_images
from matplotlib import pyplot as plt
from os.path import dirname, join
import numpy.random as npr
import numpy as np
import os
import sys
from iminuit import Minuit

class image_comparison:
    def __init__(self, baseline):

        baselineimage = join(dirname(__file__),'baseline',baseline)
        actualimage = join(os.getcwd(),'actual',baseline)

        self.baseline = baseline
        self.baselineimage = baselineimage
        self.actualimage = actualimage

        try:
            os.makedirs(dirname(actualimage))
        except OSError as e:
            pass

    def setup(self):
        from matplotlib import rcParams, rcdefaults, use
        use('Agg', warn=False) # use Agg backend for these tests

        # These settings *must* be hardcoded for running the comparison
        # tests and are not necessarily the default values as specified in
        # rcsetup.py
        rcdefaults() # Start with all defaults
        rcParams['font.family'] = 'Bitstream Vera Sans'
        rcParams['text.hinting'] = False
        rcParams['text.hinting_factor'] = 8
        rcParams['text.antialiased'] = False


    def test(self):
        #compare_images
        self.setup()
        x = compare_images(self.baselineimage, self.actualimage, 0.005)
        if x is not None:
            print x
            assert x is None


    def __call__(self, f):
        def tmp():
            f()
            plt.savefig(self.actualimage)
            return self.test()
        tmp.__name__ = f.__name__
        return tmp


@image_comparison('draw_pdf.png')
def test_draw_pdf():
    plt.figure()
    f = gaussian
    draw_pdf(f, {'mean':1.,'sigma':2.}, bound=(-10,10))


@image_comparison('draw_pdf_linear.png')
def test_draw_pdf_linear():
    plt.figure()
    f = linear
    draw_pdf(f, {'m':1.,'c':2.}, bound=(-10,10))


@image_comparison('draw_compare_hist_gaussian.png')
def test_draw_compare_hist():
    plt.figure()
    npr.seed(0)
    data = npr.randn(10000)
    edges = np.linspace(-5,5,100)
    f = gaussian
    draw_compare_hist(f, {'mean':0., 'sigma':1.}, data, normed=True)


@image_comparison('draw_compare_hist_no_norm.png')
def test_draw_compare_hist_no_norm():
    plt.figure()
    npr.seed(0)
    data = npr.randn(10000)
    edges = np.linspace(-5, 5, 100)
    f = Extended(gaussian)
    draw_compare_hist(f, {'mean':0., 'sigma':1., 'N':10000}, data, normed=False)


@image_comparison('draw_ulh.png')
def test_draw_ulh():
    npr.seed(0)
    data = npr.randn(1000)
    plt.figure()
    ulh = UnbinnedLH(gaussian, data)
    ulh.draw(args=(0., 1.))


@image_comparison('draw_ulh_with_minuit.png')
def test_draw_ulh_with_minuit():
    npr.seed(0)
    data = npr.randn(1000)
    plt.figure()
    ulh = UnbinnedLH(gaussian, data)
    m = Minuit(ulh, mean=0, sigma=1)
    ulh.draw(m)


@image_comparison('draw_blh.png')
def test_draw_blh():
    npr.seed(0)
    data = npr.randn(1000)
    blh = BinnedLH(gaussian, data)
    plt.figure()
    blh.draw(args=(0., 1.))


@image_comparison('draw_blh_extend.png')
def test_draw_blh_extend():
    npr.seed(0)
    data = npr.randn(1000)
    plt.figure()
    blh = BinnedLH(Extended(gaussian), data, extended=True)
    blh.draw(args=(0., 1., 1000))


@image_comparison('draw_bx2.png')
def test_draw_bx2():
    npr.seed(0)
    data = npr.randn(1000)
    plt.figure()
    blh = BinnedChi2(Extended(gaussian), data)
    blh.draw(args=(0., 1., 1000))


@image_comparison('draw_x2reg.png')
def test_draw_x2reg():
    npr.seed(0)
    x = np.linspace(0,1,100)
    y = 10.*x+npr.randn(100)
    err = np.array([1]*100)
    plt.figure()
    blh = Chi2Regression(linear, x, y, err)
    blh.draw(args=(10.,0.))

@image_comparison('draw_ulh_with_parts.png')
def test_ulh_with_parts():
    npr.seed(0)
    data = npr.randn(10000)
    shifted = data+3.
    data = np.append(data, [shifted])
    print len(data)
    plt.figure()
    g1 = rename(gaussian,['x', 'lmu', 'lsigma'])
    g2 = rename(gaussian,['x', 'rmu', 'rsigma'])
    allpdf = AddPdfNorm(g1, g2)
    ulh = UnbinnedLH(allpdf, data)
    ulh.draw(args=(0, 1, 3, 1, 0.5), parts=True)

@image_comparison('draw_blh_with_parts.png')
def test_blh_with_parts():
    npr.seed(0)
    data = npr.randn(10000)
    shifted = data+3.
    data = np.append(data, [shifted])
    print len(data)
    plt.figure()
    g1 = rename(gaussian,['x', 'lmu', 'lsigma'])
    g2 = rename(gaussian,['x', 'rmu', 'rsigma'])
    allpdf = AddPdfNorm(g1, g2)
    blh = BinnedLH(allpdf, data)
    blh.draw(args=(0, 1, 3, 1, 0.5), parts=True)


@image_comparison('draw_bx2_with_parts.png')
def test_bx2_with_parts():
    npr.seed(0)
    data = npr.randn(10000)
    shifted = data+3.
    data = np.append(data, [shifted])
    plt.figure()
    g1 = Extended(rename(gaussian,['x', 'lmu', 'lsigma']), extname='N1')
    g2 = Extended(rename(gaussian,['x', 'rmu', 'rsigma']), extname='N2')
    allpdf = AddPdf(g1, g2)
    bx2 = BinnedChi2(allpdf, data)
    bx2.draw(args=(0, 1, 10000, 3, 1, 10000), parts=True)

@image_comparison('draw_simultaneous.png')
def test_draw_simultaneous():
    npr.seed(0)
    data = npr.randn(10000)
    shifted = data+3.
    plt.figure()
    g1 = rename(gaussian,['x', 'lmu', 'sigma'])
    g2 = rename(gaussian,['x', 'rmu', 'sigma'])
    ulh1 = UnbinnedLH(g1, data)
    ulh2 = UnbinnedLH(g2, shifted)
    sim = SimultaneousFit(ulh1,ulh2)
    sim.draw(args=(0, 1, 3))



import matplotlib
matplotlib.use('Agg', warn=False)
from nose.tools import *
import numpy.random as npr
import numpy as np
from probfit.nputil import mid
from probfit.pdf import crystalball, gaussian
from probfit.functor import Normalized
from probfit.toy import gen_toy, gen_toyn
from probfit.util import describe
from probfit._libstat import compute_chi2
from probfit.nputil import vector_apply
from probfit.costfunc import BinnedLH

def test_gen_toy():
    npr.seed(0)
    bound = (-1,2)
    ntoy = 100000
    toy = gen_toy( crystalball,ntoy, bound=bound,
        alpha=1., n=2., mean=1., sigma=0.3, quiet=False)

    assert_equal(len(toy), ntoy)

    htoy, bins = np.histogram(toy, bins=1000, range=bound)

    ncball = Normalized(crystalball,bound)

    f = lambda x: ncball(x, 1., 2., 1., 0.3)

    expected = vector_apply(f, mid(bins))*ntoy*(bins[1]-bins[0])
    #print htoy[:100]
    #print expected[:100]

    htoy = htoy*1.0
    err = np.sqrt(expected)

    chi2 = compute_chi2(htoy, expected, err)

    print chi2, len(bins), chi2/len(bins)

    assert(0.9<(chi2/len(bins))<1.1)

def test_gen_toy2():
    pdf = gaussian
    npr.seed(0)
    toy = gen_toy(pdf,10000,(-5,5),mean=0,sigma=1)
    binlh = BinnedLH(pdf,toy,bound=(-5,5),bins=100)
    lh = binlh(0.,1.)
    for x in toy:
        assert(x < 5)
        assert(x>=-5)
    assert_equal(len(toy),10000)
    assert(lh/100. < 1.)



# -*- coding: utf-8 -*-
# <nbformat>3.0</nbformat>

# <headingcell level=1>

# dist_fit Basic Tutorial

# <codecell>

from probfit import *
import numpy as np
import iminuit
import UserDict
from math import sqrt, exp, pi

# <headingcell level=2>

# Lets start with a simple simple straight line

# <rawcell>

# We can't really call this a fitting package without being able to fit a straight line, right?

# <codecell>

#lets make s straight line with gaussian(mu=0,sigma=1) error
x = linspace(0,10,20) 
y = 3*x+15+ randn(20)
err = np.array([1]*20)
errorbar(x,y,err,fmt='.');

# <codecell>

#lets define our line
#first argument has to be independent variable
#arguments after that are shape parameters
def line(x,m,c): #define it to be parabolic or whatever you like
    return m*x+c
#We can make it faster but for this example this is plentily fast.
#We will talk about speeding things up later(you will need cython)

# <codecell>

describe(line)

# <codecell>

#cost function
chi2 = Chi2Regression(line,x,y,err)

# <codecell>

#Chi^2 regression is just a callable object nothing special about it
describe(chi2)

# <codecell>

#minimize it
#yes it gives you a heads up that you didn't give it initial value
#we can ignore it for now
minimizer = RTMinuit.Minuit(chi2) #see RTMinuit tutorial on how to give initial value/range/error
minimizer.migrad(); #very stable robust minimizer
#you can look at your terminal to see what it is doing

# <codecell>

#lets see our results
print minimizer.values
print minimizer.errors

# <codecell>

#or a pretty printing
minimizer.html_results()
#you can also call display(minimizer.html_results()) if this is in the middle of your code

# <rawcell>

# Parabolic error
# is calculated using the second derivative at the minimum
# This is good in most cases where the uncertainty is symmetric not much correlation
# exists. Migrad usually got this accurately but if you want ot be sure
# call minimizer.hesse() after migrad
# Minos Error
# is obtained by scanning chi^2 and likelihood profile and find the the point
# where chi^2 is increased by 1 or -ln likelihood is increased by 0.5
# This error is generally asymmetric and is correct in all case.
# This is quite computationally expensive if you have many parameter.
# call minimizer.minos('variablename') after migrad to get this error

# <codecell>

#let's visualize our line
chi2.draw(minimizer)
#looks good

# <codecell>

#Sometime we want the error matrix
print 'error matrix:\n', minimizer.error_matrix()
#or correlation matrix
print 'correlation matrix:\n', minimizer.error_matrix(correlation=True)
#or a pretty html
#despite the method named error_matrix it's actually a correlation matrix
display(minimizer.html_error_matrix())

# <headingcell level=2>

# Simple gaussian distribution fit

# <rawcell>

# In high energy physics, we usually want to fit a distribution to a histogram. Let's look at simple gaussian distribution

# <codecell>

#First lets make our data
data = randn(10000)*4+1
#sigma = 4 and mean = 1
hist(data,bins=100,histtype='step');

# <rawcell>

# Here is our PDF/model

# <codecell>

#normalized gaussian
def myPDF(x,mu,sigma):
    return 1/sqrt(2*pi)/sigma*exp(-(x-mu)**2/2./sigma**2)

# <codecell>

#build your cost function here we use binned likelihood
cost = BinnedLH(myPDF,data)

# <codecell>

#Let's fit
minimizer = RTMinuit.Minuit(cost,sigma=3.) #notice here that we give initial value to sigma

#up parameter determine where it determine uncertainty
#1(default) for chi^2 and 0.5 for likelihood
minimizer.set_up(0.5)
minimizer.migrad() #very stable minimization algorithm
#like in all binned fit with long zero tail. It will have to do something about the zero bin
#dist_fit handle them gracefully but will give you a head up

# <codecell>

#let's see the result
print 'Value:', minimizer.values
print 'Error:', minimizer.errors

# <codecell>

#That printout can get out of hand quickly
display(minimizer.html_results())
#and correlation matrix
#will not display well in firefox(tell them to fix writing-mode:)
display(minimizer.html_error_matrix()) 

# <codecell>

#you can see if your fit make any sense too
cost.draw(minimizer)#uncertainty is given by symetric poisson

# <codecell>

#how about making sure the error making sense
draw_contour(cost,minimizer,'mu')

# <codecell>

#2d contour error
#you can notice that it takes sometime to draw
#we will this is because our PDF is defined in Python
#we will show how to speed this up later
draw_contour2d(cost,minimizer,'mu','sigma');

# <headingcell level=2>

# How about Chi^2

# <rawcell>

# Let's explore another popular cost function chi^2

# <codecell>

#we will use the same data as in the previous example
np.random.seed(0)
data = randn(10000)*4+1
#sigma = 4 and mean = 1
hist(data,bins=100,histtype='step');

# <codecell>

#And the same PDF: normalized gaussian
def myPDF(x,mu,sigma):
    return 1/sqrt(2*pi)/sigma*exp(-(x-mu)**2/2./sigma**2)

# <codecell>

#binned chi^2 fit only makes sense for extended fit
extended_pdf = Extend(myPDF)

# <codecell>

#very useful method to look at function signature
describe(extended_pdf) #you can look at what your pdf means

# <codecell>

#Chi^2 distribution fit is really bad for distribution with long tail
#since when bin count=0... poisson error=0 and blows up chi^2
#so give it some range
chi2 = BinnedChi2(extended_pdf,data,range=(-7,10))
minimizer = RTMinuit.Minuit(chi2,sigma=1)
minimizer.migrad()

# <codecell>

chi2.draw(minimizer)

# <codecell>

#and usual deal
display(minimizer.html_results())
display(minimizer.html_error_matrix()) 

# <headingcell level=2>

# Unbinned Likelihood and How to speed things up

# <rawcell>

# Unbinned likelihood is computationally very very expensive.
# It's now a good time that we talk about how to speed things up with cython

# <codecell>

#same data
np.random.seed(0)
data = randn(10000)*4+1
#sigma = 4 and mean = 1
hist(data,bins=100,histtype='step');

# <codecell>

#We want to speed things up with cython
#load cythonmagic
%load_ext cythonmagic

# <codecell>

%%cython
cimport cython
from libc.math cimport exp,M_PI,sqrt
#same gaussian distribution but now written in cython
@cython.binding(True)#IMPORTANT:this tells cython to dump function signature too
def cython_PDF(double x,double mu,double sigma):
    #these are c add multiply etc not python so it's fast
    return 1/sqrt(2*M_PI)/sigma*exp(-(x-mu)**2/2./sigma**2)

# <codecell>

#cost function 
ublh = UnbinnedML(cython_PDF,data)
minimizer = RTMinuit.Minuit(ublh,sigma=2.)
minimizer.set_up(0.5)#remember this is likelihood
minimizer.migrad()#yes amazingly fast
ublh.show(minimizer)
display(minimizer.html_results())
display(minimizer.html_error_matrix()) 

# <codecell>

#remember how slow it was?
#now it's super fast(and it's even unbinned likelihood)
draw_contour(ublh,minimizer,'mu')

# <codecell>

#but you really don't have to write your own gaussian 
#there are tons of builtin function written in cython for you
print describe(gaussian)
print type(gaussian)

# <codecell>

ublh = UnbinnedML(gaussian,data)
minimizer = RTMinuit.Minuit(ublh,sigma=2.)
minimizer.set_up(0.5)#remember this is likelihood
minimizer.migrad()#yes amazingly fast
ublh.show(minimizer)
display(minimizer.html_results())
display(minimizer.html_error_matrix()) 

# <headingcell level=2>

# But... We can't normalize everything analytically and how to generate toy sample from PDF

# <rawcell>

# When fitting distribution to a PDF. One of the common problem that we run into is normalization.
# Not all function is analytically integrable on the range of our interest.
# Let's look at crystal ball function

# <codecell>

#lets first generate a crystal ball sample
#dist_fit has builtin toy generation capability
#lets introduce crystal ball function
#http://en.wikipedia.org/wiki/Crystal_Ball_function
#it's simply gaussian with power law tail
#normally found in energy deposited in crystals
#impossible to normalize analytically
#and normalization will depend on shape parameters
describe(crystalball)

# <codecell>

np.random.seed(0)
bound = (-1,2)
data = gen_toy(crystalball,10000,bound=bound,alpha=1.,n=2.,mean=1.,sigma=0.3,quiet=False)
#quiet = False tells it to plot out original function
#toy histogram and poisson error from both orignal distribution and toy

# <codecell>

#To fit this we need to tell normalized our crystal ball PDF
#this is done with trapezoid rule with simple cache mechanism
#can be done by Normalize functor
ncball = Normalize(crystalball,bound)
#this can also bedone with declerator
#@normalized_function(bound)
#def myPDF(x,blah):
#    return blah
print 'unnorm:', crystalball(1.0,1,2,1,0.3)
print '  norm:', ncball(1.0,1,2,1,0.3)

# <codecell>

#it has the same signature as the crystalball
describe(ncball)

# <codecell>

#now we can fit as usual
ublh = UnbinnedML(ncball,data)
minimizer = RTMinuit.Minuit(ublh,
    alpha=1.,n=2.1,mean=1.2,sigma=0.3)
minimizer.set_up(0.5)#remember this is likelihood
minimizer.migrad()#yes amazingly fast Normalize is written in cython
ublh.show(minimizer)
#crystalball function is nortorious for its sensitivity on n parameter
#dist_fit give you a heads up where it might have float overflow

# <rawcell>

# Bonus: one thing we normally ask about asymmetric peaking distribution like this is full-width-half-max
# http://en.wikipedia.org/wiki/Full_width_at_half_maximum

# <codecell>

lw,rw = fwhm_f(ncball,(-1,2),minimizer.args)

# <codecell>

ublh.draw(minimizer)
axvspan(lw,rw,alpha=0.2,color='green');

# <headingcell level=2>

# What if things went wrong

# <codecell>

#crystalball is nortoriously sensitive to initial parameter
#now it is a good time to show what happen when things...doesn't fit
ublh = UnbinnedML(ncball,data)
minimizer = RTMinuit.Minuit(ublh)#NO initial value
minimizer.set_up(0.5)#remember this is likelihood
minimizer.migrad()#yes amazingly fast
#Remember there is a heads up

# <codecell>

ublh.show(minimizer)#it bounds to fails

# <codecell>

minimizer.migrad_ok(), minimizer.matrix_accurate()
#checking these two method give you a good sign

# <codecell>

#fix it by giving it initial value/error/limit or fixing parameter see RTMinuit Tutorial
#now we can fit as usual
ublh = UnbinnedML(ncball,data)
minimizer = RTMinuit.Minuit(ublh,
    alpha=1.,n=2.1,mean=1.2,sigma=0.3)
minimizer.set_up(0.5)#remember this is likelihood
minimizer.migrad()#yes amazingly fast. Normalize is written in cython
ublh.show(minimizer)

# <headingcell level=2>

# How do we guess initial value?

# <rawcell>

# This is a hard question but visualization can helps us

# <codecell>

besttry = try_uml(ncball,data,alpha=1.,n=2.1,mean=1.2,sigma=0.3)

# <codecell>

#or you can try multiple
#too many will just confuse you
besttry = try_uml(ncball,data,alpha=1.,n=2.1,mean=[1.2,1.1],sigma=[0.3,0.5])
print besttry #and you can find which one give you minimal unbinned likelihood

# <headingcell level=2>

# Extended Fit: 2 Gaussian with Polynomial Background

# <codecell>

peak1 = randn(3000)*0.2
peak2 = randn(5000)*0.1+4
bg = gen_toy(lambda x : (x+2)**2, 20000,(-2,5))
all_data = np.concatenate([peak1,peak2,bg])
hist((peak1,peak2,bg,all_data),bins=200,histtype='step',range=(-2,5));

# <codecell>

%%cython
cimport cython
from dist_fit import Normalize, gaussian

@cython.binding(True)
def poly(double x,double a,double b, double c):
    return a*x*x+b*x+c

#remember linear function is not normalizable for -inf ... +inf
nlin = Normalize(poly,(-1,5))

#our extended PDF for 3 types of signal
@cython.binding(True)
def myPDF(double x, 
    double a, double b, double c, double nbg,
    double mu1, double sigma1, double nsig1,
    double mu2, double sigma2, double nsig2):

    cdef double NBG = nbg*nlin(x,a,b,c)
    cdef double NSIG1 = nsig1*gaussian(x,mu1,sigma1) 
    cdef double NSIG2 = nsig2*gaussian(x,mu2,sigma2)
    return NBG + NSIG1 + NSIG2

# <codecell>

print describe(myPDF)

# <codecell>

#lets use what we just learned
#for complicated function good initial value(and initial step) is crucial
#if it doesn't converge try play with initial value and initial stepsize(error_xxx)
besttry = try_binlh(myPDF,all_data, 
    a=1.,b=2.,c=4.,nbg=20000.,
    mu1=0.1,sigma1=0.2,nsig1=3000.,
    mu2=3.9,sigma2=0.1,nsig2=5000.,extended=True, bins=300, bound=(-1,5) )
print besttry

# <codecell>

binlh = BinnedLH(myPDF,all_data, bins=200, extended=True, range=(-1,5))
#too lazy to type initial values from what we try?
#use keyword expansion **besttry
#need to put in initial step size for mu1 and mu2 
#with error_mu* otherwise it won't converge(try it yourself)
minimizer = RTMinuit.Minuit(binlh, error_mu1=0.1, error_mu2=0.1, **besttry)

# <codecell>

minimizer.migrad()

# <codecell>

binlh.show(minimizer)

# <codecell>

display(minimizer.html_results())
minimizer.html_error_matrix()

# <headingcell level=2>

# Advance: Custom cost function and Simultaneous Fit

# <rawcell>

# Sometimes, what we want to fit is the sum of likelihood /chi^2 of two PDF sharing some parameters.
# dist_fit doesn't provied a built_in facility to do this but it can be built easily.
# 
# In this example, we will fit two gaussian distribution where we know that the width are the same
# but the peak is at different places.

# <codecell>

np.random.seed(0)
#Lets make two gaussian
data1 = randn(10000)+3
data2 = randn(10000)-2
hist([data1,data2],bins=100,histtype='step',label=['data1','data2']);

# <codecell>

#remember this is nothing special about builtin cost function
#except some utility function like draw and show
ulh1 = UnbinnedML(gaussian,data1)
ulh2 = UnbinnedML(gaussian,data2)
print describe(ulh1)
print describe(ulh2)

# <codecell>

#you can also use cython to do this
class CustomCost:
    def __init__(self,pdf1,data1,pdf2,data2):
        self.ulh1 = UnbinnedML(pdf1,data1)
        self.ulh2 = UnbinnedML(pdf2,data2)
    #this is the important part you need __call__ to calculate your cost
    #in our case it's sum of likelihood with sigma
    def __call__(self,mu1,mu2,sigma):
        return self.ulh1(mu1,sigma)+self.ulh2(mu2,sigma)

# <codecell>

simul_lh = CustomCost(gaussian,data1,gaussian,data2)

# <codecell>

minimizer = RTMinuit.Minuit(simul_lh,sigma=0.5)
minimizer.set_up(0.5)#remember it's likelihood
minimizer.migrad()

# <codecell>

display(minimizer.html_results())
display(minimizer.html_error_matrix())
results = minimizer.values

# <codecell>

draw_compare_hist(gaussian,[results['mu1'],results['sigma']],data1,normed=True);
draw_compare_hist(gaussian,[results['mu2'],results['sigma']],data2,normed=True);




from iminuit.util import describe #use iminuit describe..

def parse_arg(f,kwd,offset=0):
    """
    convert dictionary of keyword argument and value to positional argument
    equivalent to::

        vnames = describe(f)
        return tuple([kwd[k] for k in vnames[offset:]])

    """
    vnames = describe(f)
    return tuple([kwd[k] for k in vnames[offset:]])

def remove_prefix(s, prefix):
    if prefix is None: return s
    if s.startswith(prefix+'_'):
        l = len(prefix)+1
        return s[l:]
    else:
        return s



cimport numpy as np

cpdef double csum(np.ndarray x)

cpdef np.ndarray[np.double_t] _vector_apply(f,np.ndarray[np.double_t] x,tuple arg)

cpdef double xlogyx(double x,double y)

cpdef double wlogyx(double w,double y, double x)

cpdef double integrate1d(f, tuple bound, int nint, tuple arg=*) except *

cpdef double integrate1d_with_edges(f,np.ndarray edges, double bw, tuple arg) except *

#these are performance critical code
cpdef double compute_bin_lh_f(f,
                    np.ndarray[np.double_t] edges,
                    np.ndarray[np.double_t] h, #histogram,
                    np.ndarray[np.double_t] w2,
                    double N, #sum of h
                    tuple arg, double badvalue,
                    bint extend, bint use_sumw2) except *

cpdef double compute_nll(f,np.ndarray data,w,arg,double badvalue) except *


cpdef double compute_chi2(np.ndarray[np.double_t] actual,
                        np.ndarray[np.double_t] expected,
                        np.ndarray[np.double_t] err) except *

cpdef double compute_chi2_f(f,
                    np.ndarray[np.double_t] x,
                    np.ndarray[np.double_t] y,
                    np.ndarray[np.double_t] error,
                    np.ndarray[np.double_t] weights,
                    tuple arg) except *

cpdef double compute_bin_chi2_f(f,
                np.ndarray[np.double_t] x,np.ndarray[np.double_t] y,
                np.ndarray[np.double_t]error,np.ndarray[np.double_t] binwidth,
                np.ndarray[np.double_t]weights,tuple arg) except *

cpdef compute_cdf(np.ndarray[np.double_t] pdf, np.ndarray[np.double_t] x)

#invert cdf useful for making toys
cpdef invert_cdf(np.ndarray[np.double_t] r,
                np.ndarray[np.double_t] cdf,
                np.ndarray[np.double_t] x)



#cython: embedsignature=True
cimport cython
from cpython cimport PyFloat_AsDouble, PyTuple_GetItem, PyTuple_GET_ITEM,\
                     PyObject, PyTuple_SetItem, PyTuple_SET_ITEM,\
                     PyTuple_New, Py_INCREF

import numpy as np
cimport numpy as np
from warnings import warn
from probfit_warnings import SmallIntegralWarning
from _libstat cimport integrate1d_with_edges, _vector_apply
from funcutil import FakeFuncCode, merge_func_code, FakeFunc
from util import describe


cpdef tuple construct_arg(tuple arg, np.ndarray[np.int_t] fpos):
    cdef int size = fpos.shape[0]
    cdef int i,itmp
    cdef np.int_t* fposdata = <np.int_t*>fpos.data
    cdef tuple ret = PyTuple_New(size)
    cdef object tmpo
    for i in range(size):
        itmp = fposdata[i]
        tmpo = <object>PyTuple_GET_ITEM(arg, itmp)
        Py_INCREF(tmpo)
        #Py_INCREF(tmpo) #first one for the case second one for the steal
        PyTuple_SET_ITEM(ret, i, tmpo)
    return ret

#TODO: optimize this may be check id() instead of actual comparison?
cpdef bint fast_tuple_equal(tuple t1, tuple t2 , int t2_offset) except *:
    #t=last_arg
    #t2=arg
    cdef double tmp1,tmp2
    cdef int i,ind
    cdef int tsize = len(t2)-t2_offset
    cdef bint ret = 0
    cdef double precision = 1e-16
    if len(t1) ==0 and tsize==0:
        return 1

    for i in range(tsize):
        ind = i+t2_offset
        tmp1 = PyFloat_AsDouble(<object>PyTuple_GetItem(t1,i))
        tmp2 =  PyFloat_AsDouble(<object>PyTuple_GetItem(t2,ind))
        ret = abs(tmp1-tmp2) < precision
        if not ret: break

    return ret

cdef class Convolve:#with gy cache
    """
    Make convolution from supplied **f** and **g**. If your functions are
    analytically convolutable you will be better off implementing
    analytically. This functor is implemented using numerical integration with
    bound there are numerical issue that will, in most cases, lightly affect
    normalization.s

    Argument from **f** and **g** is automatically merge by name. For example,

    ::

        f = lambda x, a, b, c: a*x**2+b*x+c
        g = lambda x, a, sigma: gaussian(x,a,sigma)
        h = Convolve(f,g,gbound)
        describe(h)#['x','a','b','c','sigma']

        #h is equivalent to
        def h_equiv(x, a, b, c, sigma):
            return Integrate(f(x-t, a, b, c)*g(x, a, b, c), t_range=gbound)

    .. math::

        \\text{Convolve(f,g)}(t, arg \ldots) =
                \int_{\\tau \in \\text{gbound}}
                f(t-\\tau, arg \ldots)\\times g(t, arg\ldots) \, \mathrm{d}\\tau

    **Argument**

        - **f** Callable object, PDF.
        - **g** Resolution function.
        - **gbound** Bound of the resolution. Supplied something such that
          g(x,*arg) is near 0 at the edges. Current implementation is
          multiply reverse slide add straight up from the definition so this
          bound is important. Overbounding is recommended.
        - **nbins** Number of bins in multiply reverse slide add. Default(1000)

    .. note::

        You may be worried about normalization. By the property of convolution:

        .. math::

            \int_{\mathbf{R}^d}(f*g)(x) \, dx=\left(\int_{\mathbf{R}^d}f(x) \, dx\\right)\left(\int_{\mathbf{R}^d}g(x) \, dx\\right).

        This means if you convolute two normalized distributions you get back
        a normalized distribution. However, since we are doing numerical
        integration here we will be off by a little bit.

    """
    cdef int numbins
    cdef tuple gbound
    cdef double bw
    cdef int nbg
    cdef f #original f
    cdef g #original g
    cdef np.ndarray fpos#position of argument in f
    cdef np.ndarray gpos#position of argument in g
    cdef public object func_code
    cdef public object func_defaults
    cdef tuple last_garg
    cdef np.ndarray gy_cache

    #g is resolution function gbound need to be set so that the end of g is zero
    def __init__(self, f, g, gbound, nbins=1000):
        self.set_gbound(gbound,nbins)
        self.func_code, [self.fpos, self.gpos] = merge_func_code(f,g,skip_first=True)
        self.func_defaults = None
        self.f = f
        self.g = g

    def set_gbound(self,gbound,nbins):
        self.last_garg = None
        self.gbound,self.nbg = gbound,nbins
        self.bw = 1.0*(gbound[1]-gbound[0])/nbins
        self.gy_cache = None

    def __call__(self,*arg):
        #skip the first one
        cdef int iconv
        cdef double ret = 0
        cdef np.ndarray[np.double_t] gy,fy

        tmp_arg = arg[1:]
        x=arg[0]
        garg = list()

        for i in self.gpos: garg.append(arg[i])
        garg = tuple(garg[1:])#dock off the independent variable

        farg = list()
        for i in self.fpos: farg.append(arg[i])
        farg = tuple(farg[1:])

        xg = np.linspace(self.gbound[0],self.gbound[1],self.nbg)

        gy = None
        #calculate all the g needed
        if garg==self.last_garg:
            gy = self.gy_cache
        else:
            gy = _vector_apply(self.g, xg, garg)
            self.gy_cache=gy

        #now prepare f... f needs to be calculated and padded to f-gbound[1] to f+gbound[0]
        #yep this is not a typo because we are "reverse" sliding g onto f so we need to calculate f from
        # f-right bound of g to f+left bound of g
        fbound = x-self.gbound[1], x-self.gbound[0] #yes again it's not a typo
        xf = np.linspace(fbound[0], fbound[1], self.nbg)#yep nbg
        fy = _vector_apply(self.f, xf, farg)
        #print xf[:100]
        #print fy[:100]
        #now do the inverse slide g and f
        for iconv in range(self.nbg):
            ret += fy[iconv]*gy[self.nbg-iconv-1]

        #now normalize the integral
        ret*=self.bw

        return ret


cdef class Extended:
    """
    Transformed given **f** into extended from.

    ::

        def f(x,mu,sigma):
            return gaussian(x,mu,sigma)
        g = Extended(f) #g is equivalent to N*f(x,mu,sigma)
        describe(g) #('x','mu','sigma','N')

        #g is equivalent to
        def g_equiv(x, mu, sigma, N):
            return N*f(x, mu, sigma)

    **Arguments**
        - **f** call object. PDF.
        - **extname** optional string. Name of the extended parameter. Default
          `'N'`

    """
    cdef f
    cdef public func_code
    cdef public func_defaults
    def __init__(self,f,extname='N'):
        self.f = f
        if extname in describe(f):
            raise ValueError('%s is already taken pick something else for extname')
        self.func_code = FakeFuncCode(f,append=extname)
        #print self.func_code.__dict__
        self.func_defaults=None
    def __call__(self,*arg):
        cdef double N = arg[-1]
        cdef double fval = self.f(*arg[:-1])
        return N*fval

cdef class AddPdf:
    """
    Directly add PDF without normalization nor factor.
    Parmeters are merged by names.

    ::

        def f(x, a, b, c):
            return do_something(x,a,b,c)
        def g(x, d, a, e):
            return do_something_else(x, d, a, e)

        h = AddPdf(f, g)# you can do AddPdf(f, g, h)
        #h is equivalent to
        def h_equiv(x, a, b, c, d, e):
            return f(x, a, b, c) + g(x, d, a, e)

    **Arguments**

        - **prefix** array of prefix string with length equal to number of
          callable object passed. This allows you to add two PDF without having
          parameters from the two merge. ::

                h2 = AddPdf(f, g, prefix=['f','g'])
                #h is equivalent to
                def h2_equiv(x, f_a, f_b, f_c, g_d, g_a, g_e):
                    return f(x, f_a, f_b, f_c) + g(x, g_d, g_a, g_e)

    """
    #FIXME: cache each part if called with same parameter
    cdef public object func_code
    cdef public object func_defaults
    cdef int arglen
    cdef list allpos
    cdef tuple allf
    cdef readonly int numf
    cdef np.ndarray cache
    cdef list argcache
    cdef public int hit
    cdef public int nparts

    def __init__(self,*arg,prefix=None):
        self.func_code, self.allpos = merge_func_code(*arg,prefix=prefix,skip_first=True)
        self.func_defaults=None
        self.arglen = self.func_code.co_argcount
        self.allf = arg
        self.numf = len(self.allf)
        self.argcache=[None]*self.numf
        self.cache = np.zeros(self.numf)
        self.hit = 0

    def __call__(self,*arg):
        cdef tuple this_arg
        cdef double ret = 0.
        cdef double tmp = 0.
        cdef int i
        cdef np.ndarray thispos
        for i in range(self.numf):
            thispos = self.allpos[i]
            this_arg = construct_arg(arg,thispos)
            if self.argcache[i] is not None and fast_tuple_equal(this_arg,self.argcache[i],0):
                tmp = self.cache[i]
                self.hit+=1
            else:
                tmp = self.allf[i](*this_arg)
                self.argcache[i]=this_arg
                self.cache[i]=tmp
            ret+=tmp
        return ret


    def parts(self):
        return [self._part(i) for i in range(self.numf)]


    def _part(self, int findex):

        def tmp(*arg):
            thispos = self.allpos[findex]
            this_arg = construct_arg(arg, thispos)
            return self.allf[findex](*this_arg)

        tmp.__name__ = getattr(self.allf[findex],'__name__','unnamedpart')
        ret = FakeFunc(tmp)
        ret.func_code = self.func_code
        return ret


    def eval_parts(self,*arg):
        cdef tuple this_arg
        cdef double tmp = 0.
        cdef int i
        cdef list ref
        cdef np.ndarray thispos
        ret = list()
        for i in range(self.numf):
            thispos = self.allpos[i]
            this_arg = construct_arg(arg,thispos)
            if self.argcache[i] is not None and fast_tuple_equal(this_arg,self.argcache[i],0):
                tmp = self.cache[i]
                self.hit+=1
            else:
                tmp = self.allf[i](*this_arg)
                self.argcache[i]=this_arg
                self.cache[i]=tmp
            ret.append(tmp)
        return tuple(ret)

cdef class AddPdfNorm:
    """
    Add PDF with normalization factor. Parameters are merged by name.

    ::

        def f(x, a, b, c):
            return do_something(x, a, b, c)
        def g(x, d, a, e):
            return do_something_else(x, d, a, e)
        def p(x, b, a, c):
            return do_something_other_thing(x,b,a,c)
        h = Add2PdfNorm(f, g, p)

        #h is equivalent to
        def h_equiv(x, a, b, c, d, e, f_0, f_1):
            return f_0*f(x, a, b, c)+ \\
                   f_1*g(x, d, a, e)+
                   (1-f_0-f_1)*p(x, b, a, c)

    **Arguments**
        - **facname** optional list of factor name of length=. If None is given
          factor name is automatically chosen to be `f_0`, `f_1` etc.
          Default None.
        - **prefix** optional prefix list to prefix arguments of each function.
          Default None.


    """
    #FIXME: cache each part if called with same parameter
    cdef public object func_code
    cdef public object func_defaults
    cdef int arglen
    cdef int normalarglen
    cdef tuple allf
    cdef readonly int numf
    #cdef f
    #cdef g
    cdef list allpos
    #cdef np.ndarray fpos
    #cdef np.ndarray gpos
    cdef public int nparts

    def __init__(self, *arg ,facname=None, prefix=None):

        self.func_code, self.allpos = merge_func_code(*arg,
            prefix=prefix,skip_first=True)

        if facname is not None and len(facname)!=len(arg)-1:
            raise(RuntimeError('length of facname and arguments must satisfy len(facname)==len(arg)-1'))

        self.normalarglen = self.func_code.co_argcount
        #TODO check name collisions here

        if facname is None:
            #automatic naming
            facname = ['f_%d'%i for i in range(len(arg)-1)]

        for fname in facname:
            self.func_code.append(fname)

        self.arglen = self.func_code.co_argcount
        self.func_defaults=None
        self.allf = arg
        self.numf = len(arg)


    def __call__(self,*arg):
        cdef ret = 0.
        cdef tuple farg
        cdef double allfac = 0.
        cdef double fac = 0.
        cdef int findex = 0

        for findex in range(self.numf):
            if findex!=self.numf-1: #not the last one
                fac = arg[self.normalarglen+findex]
                allfac += fac
            else: #last one
                fac = 1-allfac
            farg = construct_arg(arg,self.allpos[findex])
            ret += fac*self.allf[findex](*farg)
        return ret

    def parts(self):
        return [self._part(i) for i in range(self.numf)]

    def _part(self, int findex):
        #FIXME make this faster. How does cython closure work?
        def tmp(*arg):
            if findex!=self.numf-1: #not the last one
                fac = arg[self.normalarglen+findex]
            else: #last one
                fac = sum(arg[self.normalarglen:])
            thispos = self.allpos[findex]
            this_arg = construct_arg(arg, thispos)
            return fac*self.allf[findex](*this_arg)

        tmp.__name__ = getattr(self.allf[findex],'__name__','unnamedpart')
        ret = FakeFunc(tmp)
        ret.func_code = self.func_code
        return ret

    def eval_parts(self,*arg):

        cdef tuple farg
        cdef double allfac = 0.
        cdef double fac = 0.
        cdef int findex = 0
        cdef list ret = []

        for findex in range(self.numf):
            if findex!=self.numf-1: #not the last one
                fac = arg[self.normalarglen+findex]
                allfac += fac
            else: #last one
                fac = 1-allfac
            farg = construct_arg(arg,self.allpos[findex])
            ret.append(fac*self.allf[findex](*farg))
        return tuple(ret)


cdef class Normalized:
    """
    Transformed PDF in to a normalized version. The normalization factor is
    cached according the shape parameters(all arguments except the first one).

    ::

        def f(x, a, b, c):
            return do_something(x, a, b, c)
        g = Normalized(f, (0., 1.))
        #g is eqivalent to (shown here without cache)
        def g_equiv(x, a, b, c):
            return f(x, a, b, c)/Integrate(f(x, a, b, c), range=(0., 1.))

    **Arguments**
        - **f** function to normalized.
        - **bound** bound of the normalization.
        - **nint** optional number of pieces to integrate. Default 1000.
        - **warnfloat** optinal number of times it should warn if integral
          of the given function is really small. This usually indicate
          you bound doesn't make sense with given parameters.

    .. note::
        Integration implemented here is just a simple trapezoid rule.
        You are welcome to implement something better and submit a pull
        request.

    .. warning::
        Never reused Normalized object with different parameters in the same pdf.

        ::

            #DO NOT DO THIS
            def f(x,y,z):
                do_something(x,y,z)
            pdf1 = Normalized(f,(0,1)) #h has it's own cache

            pdf2 = rename(pdf1,['x','a','b'])#don't do this

            totalpdf = Add2PdfNorm(pdf1,pdf2)

        The reason is that Normalized has excatly one cache value. Everytime
        it's called with different parameters the cache is invalidate and it
        will recompute the integration which takes a long time. For the
        example given above, when calling totalpdf, calling to `pdf2` will
        always invalidate `pdf1` cache causing it to recompute integration
        for every datapoint `x`. The fix is easy::

            #DO THIS INSTEAD
            def f(x,y,z):
                do_something(x,y,z)
            pdf1 = Normalized(f,(0,1)) #h has it's own cache

            pdf2_temp = Normalized(f,(0,1)) #own separate cache
            pdf2 = rename(pdf2_temp,['x','a','b'])

            totalpdf = Add2PdfNorm(pdf1,pdf2)

    """
    cdef f
    cdef double norm_cache
    cdef tuple last_arg
    cdef int nint
    cdef np.ndarray edges
    #cdef np.ndarray binwidth
    cdef double binwidth
    cdef public object func_code
    cdef public object func_defaults
    cdef int ndep
    cdef int warnfloat
    cdef int floatwarned
    cdef public int hit
    def __init__(self,f,bound,nint=1000,warnfloat=1):
        self.f = f
        self.norm_cache= 1.
        self.last_arg = None
        self.nint = nint
        # normx = normx if normx is not None else np.linspace(range[0],range[1],nint)
        #         if normx.dtype!=normx.dtype:
        #             normx = normx.astype(np.float64)
        #print range
        #print normx
        self.edges = np.linspace(bound[0],bound[1],nint)
        #print self.midpoints
        self.binwidth = self.edges[1]-self.edges[0]
        self.func_code = FakeFuncCode(f)
        self.ndep = 1#TODO make the code doesn't depend on this assumption
        self.func_defaults = None #make vectorize happy
        self.warnfloat=warnfloat
        self.floatwarned=0
        self.hit=0

    def __call__(self,*arg):
        #print arg
        cdef double n
        cdef double x
        n = self._compute_normalization(*arg)
        x = self.f(*arg)
        if self.floatwarned < self.warnfloat  and n < 1e-100:
            warn(SmallIntegralWarning(str(arg)))
            self.floatwarned+=1
        return x/n

    def _compute_normalization(self, *arg):
        cdef tuple targ = arg[self.ndep:]
        #if targ == self.last_arg:#cache hit
        if self.last_arg is not None and fast_tuple_equal(targ,self.last_arg,0):
            #targ == self.last_arg:#cache hit
            #yah exact match for float since this is expected to be used
            #in vectorize which same value are passed over and over
            self.hit+=1
            pass
        else:
            self.last_arg = targ
            self.norm_cache = integrate1d_with_edges(self.f, self.edges,
                                                    self.binwidth, targ)
        return self.norm_cache




import unittest
from probfit import *
from probfit._libstat import integrate1d
from nose.tools import *
from probfit.decorator import extended, normalized

def test_describe_normal_function():
    def f(x,y,z):
        return x+y+z
    d = describe(f)
    assert_equal(list(d),['x','y','z'])


def test_Normalized():
    f = ugaussian
    g = Normalized(f,(-1,1))

    norm = integrate1d(f,(-1.,1.),1000,(0.,1.))
    assert_almost_equal(g(1.,0.,1.),f(1.,0.,1.)/norm)

def test_normalized_decorator():
    @normalized((-1,1))
    def f(x, mean, sigma):
        return ugaussian(x, mean, sigma)
    g = Normalized(ugaussian,(-1,1))
    assert_equal(describe(f), ['x','mean','sigma'])
    assert_almost_equal(g(1,0,1),f(1,0,1))

def test_Normalized_cache_hit():
    def f(x,y,z) : return 1.*(x+y+z)
    def g(x,y,z) : return 1.*(x+y+2*z)
    nf = Normalized(f,(-10.,10.))
    ng = Normalized(g,(-10.,10.))
    assert_equal(nf.hit,0)
    nf(1.,2.,3.)
    ng(1.,2.,3.)
    assert_equal(nf.hit,0)
    nf(3.,2.,3.)
    assert_equal(nf.hit,1)
    ng(1.,2.,3.)
    assert_equal(ng.hit,1)

def test_add_pdf():
    def f(x,y,z): return x+y+z
    def g(x,a,b): return 2*(x+a+b)
    def h(x,c,a): return 3*(x+c+a)

    A = AddPdf(f,g,h)
    assert_equal(tuple(describe(A)),('x','y','z','a','b','c'))

    ret = A(1,2,3,4,5,6,7)
    expected = f(1,2,3)+g(1,4,5)+h(1,6,4)
    assert_almost_equal(ret,expected)


def test_add_pdf_cache():
    def f(x,y,z): return x+y+z
    def g(x,a,b): return 2*(x+a+b)
    def h(x,c,a): return 3*(x+c+a)

    A = AddPdf(f,g,h)
    assert_equal(tuple(describe(A)), ('x','y','z','a','b','c'))

    ret = A(1,2,3,4,5,6,7)
    assert_equal(A.hit,0)
    expected = f(1,2,3)+g(1,4,5)+h(1,6,4)
    assert_almost_equal(ret,expected)

    ret = A(1,2,3,6,7,8,9)
    assert_equal(A.hit,1)
    expected = f(1,2,3)+g(1,6,7)+h(1,8,6)
    assert_almost_equal(ret,expected)


def test_extended():
    def f(x,y,z): return x+2*y+3*z
    g = Extended(f)
    assert_equal(tuple(describe(g)), ('x','y','z','N'))
    assert_equal(g(1,2,3,4), 4*(f(1,2,3)))


def test_extended_decorator():
    def f(x,y,z): return x+2*y+3*z

    @extended()
    def g(x,y,z):
        return x+2*y+3*z

    assert_equal(tuple(describe(g)), ('x','y','z','N'))
    assert_equal(g(1,2,3,4), 4*(f(1,2,3)))


def test_addpdfnorm():
    def f(x,y,z): return x+2*y+3*z
    def g(x,z,p): return 4*x+5*z+6*z
    def p(x,y,q): return 7*x+8*y+9*q

    h = AddPdfNorm(f,g)
    assert_equal(describe(h),['x', 'y', 'z', 'p', 'f_0'])

    q = AddPdfNorm(f,g,p)
    assert_equal(describe(q),['x', 'y', 'z', 'p', 'q', 'f_0', 'f_1'])

    assert_almost_equal(h(1,2,3,4,0.1),
            0.1*f(1,2,3)+0.9*g(1,3,4))

    assert_almost_equal(q(1,2,3,4,5,0.1,0.2),
            0.1*f(1,2,3)+0.2*g(1,3,4)+0.7*p(1,2,5))

def test_convolution():
    f = gaussian
    g = lambda x, mu1, sigma1 : gaussian(x, mu1, sigma1)

    h = Convolve(f,g,(-10,10),nbins=10000)
    assert_equal(describe(h),['x', 'mean', 'sigma', 'mu1', 'sigma1'])

    assert_almost_equal(h(1,0,1,1,2),0.17839457037411527)#center
    assert_almost_equal(h(-1,0,1,1,2),0.119581456625684)#left
    assert_almost_equal(h(0,0,1,1,2),0.1614180824489487)#left
    assert_almost_equal(h(2,0,1,1,2),0.1614180824489487)#right
    assert_almost_equal(h(3,0,1,1,2),0.119581456625684)#right


def test_rename():
    def f(x,y,z):
        return None
    assert_equal(describe(f), ['x','y','z'])
    g = rename(f,['x','a','b'])
    assert_equal(describe(g), ['x','a','b'])




__all__ = [
    'gen_toy',
    'gen_toyn',
]

import numpy as np
from ._libstat import compute_cdf, invert_cdf, _vector_apply
from util import describe
import numpy.random as npr
from matplotlib import pyplot as plt
from probfit_warnings import SmallIntegralWarning

def gen_toyn(f, nsample, ntoy, bound, accuracy=10000, quiet=True, **kwd):
    """
    just alias of gentoy for nample and then reshape to ntoy,nsample)
    :param f:
    :param nsample:
    :param bound:
    :param accuracy:
    :param quiet:
    :param kwd:
    :return:
    """
    return gen_toy(f, nsample*ntoy, bound, accuracy, quiet, **kwd).reshape((ntoy,nsample))

def gen_toy(f, nsample, bound, accuracy=10000, quiet=True, **kwd):
    """
    generate ntoy
    :param f:
    :param nsample:
    :param ntoy:
    :param bound:
    :param accuracy:
    :param quiet:
    :param kwd: the rest of keyword argument will be passed to f
    :return: numpy.ndarray
    """
    #based on inverting cdf this is fast but you will need to give it a reasonable range
    #unlike roofit which is based on accept reject

    vnames = describe(f)
    if not quiet:
        print vnames
    my_arg = [kwd[v] for v in vnames[1:]]
    #random number
    #if accuracy is None: accuracy=10*numtoys
    r = npr.random_sample(nsample)
    x = np.linspace(bound[0], bound[1], accuracy)
    pdf = _vector_apply(f, x, tuple(my_arg))
    cdf = compute_cdf(pdf, x)
    if cdf[-1] < 0.01:
        warn(SmallIntegralWarning('Integral for given funcition is'
            ' really low. Did you give it a reasonable range?'))
    cdfnorm = cdf[-1]
    cdf /= cdfnorm

    #now convert that to toy
    ret = invert_cdf(r, cdf, x)

    if not quiet:
        #move this to plotting
        plt.figure()
        plt.title('comparison')
        numbin = 100
        h,e = np.histogram(ret,bins=numbin)
        mp = (e[1:]+e[:-1])/2.
        err = np.sqrt(h)
        plt.errorbar(mp,h,err,fmt='.b')
        bw = e[1] - e[0]
        y = pdf * len(ret) / cdfnorm * bw
        ylow = y + np.sqrt(y)
        yhigh = y - np.sqrt(y)
        plt.plot(x, y, label='pdf', color='r')
        plt.fill_between(x, yhigh, ylow, color='g', alpha=0.2)
        plt.grid(True)
        plt.xlim(bound)
        plt.ylim(ymin=0)
    return ret




import numpy as np
from _libstat import _vector_apply


def fwhm_f(f, range, arg=None, bins=1000):

    if arg is None: arg=tuple()

    x = np.linspace(range[0], range[1], bins)
    y = _vector_apply(f, x, arg)
    imax = np.argmax(y)
    ymax = y[imax]
    rs = y[imax:]-ymax/2.0
    ls = y[:imax]-ymax/2.0
    xl = 0
    xr = 0

    il = first_neg(ls,'l')
    #print il,x[il],ls[il],x[il+1],ls[il+1]
    xl = xintercept(x[il], ls[il], x[il+1], ls[il+1])

    ir = first_neg(rs, 'r')
    #print ir,x[imax+ir],rs[ir],x[ir+1],rs[ir+1]
    xr = xintercept(x[imax+ir], rs[ir], x[imax+ir-1], rs[ir-1])

    return (xl, xr)

def xintercept_tuple(t0, t1):
    return xintercept(t0[0],t0[1],t1[0],t1[1])

def xintercept(x0, y0, x1, y1):
    m = (y1-y0)/(x1-x0)
    return -y0/m+x0

def first_neg(y, direction='r'):
    if direction == 'l':
        xlist = xrange(len(y)-1, -1, -1)
    else:
        xlist = xrange(len(y))
    ret = 0
    found = False
    for i in xlist:
        if y[i] < 0:
            ret = i
            found = True
            break
    if not found:
        raise ValueError('They are all positive what are you tying to find?')
    return ret



import unittest
from probfit.costfunc import UnbinnedLH, BinnedLH, BinnedChi2, Chi2Regression,\
                             SimultaneousFit
from probfit.pdf import gaussian, linear
from numpy.random import randn, seed
from math import log,pi,sqrt
import warnings
from iminuit.iminuit_warnings import InitialParamWarning
from probfit.util import describe
from nose.tools import *
from probfit.funcutil import rename
import numpy as np
from iminuit import Minuit

def assert_almost_equal(x,y,delta=1e-7):
    assert(y-delta < x < y+delta)

class TestFit(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter("ignore", InitialParamWarning)
        seed(0)
        self.ndata = 20000
        self.data = randn(self.ndata)
        self.analytic = self.ndata*0.5*(log(2*pi)+1)

    def test_UnbinnedLH(self):
        f = gaussian
        assert_equal(list(describe(f)), ['x','mean','sigma'])
        lh = UnbinnedLH(gaussian, self.data,)
        assert_equal(list(describe(lh)), ['mean','sigma'])
        assert_almost_equal(lh(0,1), 28188.201229348757)

    def test_BinnedLH(self):
        #write a better test... this depends on subtraction
        f = gaussian
        assert_equal(list(describe(f)), ['x','mean','sigma'])
        lh = BinnedLH(gaussian, self.data, bound=[-3,3])
        assert_equal(list(describe(lh)), ['mean','sigma'])
        assert_almost_equal(lh(0,1), 20.446130781601543)


    def test_BinnedChi2(self):
        f = gaussian
        assert_equal(list(describe(f)), ['x','mean','sigma'])
        lh = BinnedChi2(gaussian, self.data, bound=[-3,3])
        assert_equal(list(describe(lh)), ['mean','sigma'])
        assert_almost_equal(lh(0,1), 19951.005399882044)


    def test_Chi2Regression(self):
        x = np.linspace(1, 10, 10)
        y = 10*x+1
        f = linear
        assert_equal(list(describe(f)), ['x','m','c'])

        lh = Chi2Regression(f, x, y)

        assert_equal(list(describe(lh)), ['m','c'])

        assert_almost_equal(lh(10, 1), 0)

        assert_almost_equal(lh(10, 0), 10.)

    def test_simultaneous(self):
        seed(0)
        data = randn(10000)
        shifted = data+3.
        g1 = rename(gaussian, ['x', 'lmu', 'sigma'])
        g2 = rename(gaussian, ['x', 'rmu', 'sigma'])
        ulh1 = UnbinnedLH(g1,data)
        ulh2 = UnbinnedLH(g2,shifted)
        sim = SimultaneousFit(ulh1, ulh2)
        assert_equal(describe(sim),['lmu', 'sigma', 'rmu'])
        m = Minuit(sim,sigma=1.2, pedantic=False, print_level=0)
        m.migrad()
        assert_almost_equal(m.values['lmu'], 0., delta=2*m.errors['lmu'])
        assert_almost_equal(m.values['rmu'], 3., delta=2*m.errors['rmu'])
        assert_almost_equal(m.values['sigma'], 1., delta=2*m.errors['sigma'])

if __name__ == '__main__':
    unittest.main()



import numpy as np
from _libstat import _vector_apply
def mid(x):
    return (x[:-1]+x[1:])/2

def minmax(x):
    return min(x),max(x)

#for converting numpy array to double
def float2double(a):
    if a is None or a.dtype == np.float64:
        return a
    else:
        return a.astype(np.float64)

def vector_apply(f, x, *arg):
    """
    apply **f** to array **x** with given arguments fast. This is a fast
    version of::

        np.array([f(xi,*arg) for xi in x ])

    useful when you try to plot something
    """
    return _vector_apply(f,x,arg)



 # -*- coding: utf-8 -*-
#Plotting is on python since this will make it much easier to debug and adjsut
#no need to recompile everytime i change graph color....

#needs a serious refactor
from matplotlib import pyplot as plt
import numpy as np
from .nputil import mid, minmax, vector_apply
from util import parse_arg, describe
from math import sqrt, ceil, floor

def draw_simultaneous(self, minuit=None, args=None, errors=None):
    numf = len(self.allf)
    numraw = sqrt(numf)
    numcol = ceil(numraw)
    numrow = floor(numraw) if floor(numraw)*numcol>=numf else ceil(numraw)

    for i in range(numf):
        plt.subplot(numrow, numcol, i+1)
        part_args, part_errors = self.args_and_error_for(i, minuit, args, errors)
        self.allf[i].draw(args=part_args, errors=part_errors)

def _get_args_and_errors(self, minuit=None, args=None, errors=None ):
    """
    consitent algorithm to get argument and errors
    1) get it from minuit if minuit is avaialble
    2) if not get it from args and errors
    2.1) if args is dict parse it.
    3) if all else fail get it from self.last_arg
    """
    ret_arg = None
    ret_error = None
    if minuit is not None: # case 1
        ret_arg = minuit.args
        ret_error = minuit.errors
        return ret_arg, ret_error

    #no minuit specified use args and errors
    if args is not None:
        if isinstance(args, dict):
            ret_arg = parse_arg(self, args)
        else:
            ret_arg = args
    else: # case 3
        ret_arg = self.last_arg

    if errors is not None:
        ret_error = errors

    return ret_arg, ret_error


def _param_text(parameters, arg, error):
    txt = u''
    for i, (k, v) in enumerate(zip(parameters, arg)):
        txt += u'%s = %5.4g'%(k,v)
        if error is not None:
            txt += u'±%5.4g'%error[k]
        txt += u'\n'
    return txt


#from UML
def draw_ulh(self, minuit=None, bins=100, ax=None, bound=None,
            parmloc=(0.05,0.95), nfbins=500, print_par=False,
            args=None, errors=None, parts=False):

    ax = plt.gca() if ax is None else ax

    arg, error = _get_args_and_errors(self, minuit, args, errors)

    n, e, patches = ax.hist(self.data, bins=bins, weights=self.weights,
                            histtype='step', range=bound, normed=True)

    bound = (e[0],e[-1])
    draw_arg = [('lw',2)]
    if not parts: draw_arg.append(('color','r'))

    draw_pdf(self.f, arg, bound, bins=nfbins, density=True,
                    **dict(draw_arg))

    if parts:
        f_parts = getattr(self.f, 'parts', None)
        if f_parts is not None:
            for p in f_parts():
                draw_pdf(p, arg, bound, bins=nfbins, density=True)

    ax.grid(True)

    txt = _param_text(describe(self), arg, error)
    if print_par: print txt
    ax.text(parmloc[0], parmloc[1], txt, ha='left', va='top',
            transform=ax.transAxes)


#from chi2 regression
def draw_x2(self, minuit=None, ax=None, parmloc=(0.05,0.95), print_par=False,
            args=None, errors=None):

    ax = plt.gca() if ax is None else ax

    arg, error = _get_args_and_errors(self, minuit, args, errors)

    x=self.x
    y=self.y
    data_err = self.error

    if data_err is None:
        plt.plot(x,y,'+')
    else:
        plt.errorbar(x,y,data_err,fmt='.')

    draw_arg = [('lw',2)]
    draw_arg.append(('color','r'))

    draw_pdf_with_midpoints(self.f, arg, x, **dict(draw_arg))

    ax.grid(True)

    txt = _param_text(describe(self), arg, error)

    chi2 = self(*arg)
    txt+=u'chi2/ndof = %5.4g(%5.4g/%d)'%(chi2/self.ndof, chi2, self.ndof)

    if print_par: print txt

    ax.text(parmloc[0], parmloc[1], txt, ha='left', va='top',
            transform=ax.transAxes)


#from binned chi2
def draw_bx2(self, minuit=None, parmloc=(0.05,0.95), nfbins=500, ax=None,
             print_par=False, args=None, errors=None, parts=False):

    ax = ax=plt.gca() if ax is None else ax

    arg, error = _get_args_and_errors(self, minuit, args, errors)

    m = mid(self.edges)

    ax.errorbar(m, self.h, self.err, fmt='.')

    bound = (self.edges[0], self.edges[-1])

    scale = nfbins/self.bins #scale back to bins

    draw_arg = [('lw',2)]
    if not parts: draw_arg.append(('color','r'))

    draw_pdf(self.f, arg, bins=nfbins, bound=bound, density=False,
             scale=scale, **dict(draw_arg))


    if parts:
        f_parts = getattr(self.f, 'parts', None)
        if f_parts is not None:
            for p in f_parts():
                draw_pdf(p, arg, bound=bound, bins=nfbins, density=False,
                         scale=scale)

    ax.grid(True)

    txt = _param_text(describe(self), arg, error)

    chi2 = self(*arg)
    txt+=u'chi2/ndof = %5.4g(%5.4g/%d)'%(chi2/self.ndof, chi2, self.ndof)

    if print_par: print txt

    ax.text(parmloc[0], parmloc[1], txt, ha='left', va='top',
            transform=ax.transAxes)


#from binnedLH
def draw_blh(self, minuit=None, parmloc=(0.05,0.95),
                nfbins=1000, ax = None, print_par=False,
                args=None, errors=None, parts=False):
    ax = ax=plt.gca() if ax is None else ax

    arg, error = _get_args_and_errors(self, minuit, args, errors)

    m = mid(self.edges)

    if self.use_w2:
        err = np.sqrt(self.w2)
    else:
        err = np.sqrt(self.h)

    if self.extended:
        ax.errorbar(m, self.h, err, fmt='.')
    else:
        scale = 1./sum(self.h)/self.binwidth
        ax.errorbar(m, self.h*scale, err*scale, fmt='.')

    draw_arg = [('lw',2)]
    if not parts: draw_arg.append(('color','r'))
    bound = (self.edges[0], self.edges[-1])
    scale = 1. if not self.extended else nfbins/self.bins #scale back to bins
    draw_pdf(self.f, arg, bins=nfbins, bound=bound, density=not self.extended,
             scale=scale, **dict(draw_arg))


    if parts:
        f_parts = getattr(self.f, 'parts', None)
        if f_parts is not None:
            for p in f_parts():
                draw_pdf(p, arg, bins=nfbins, bound=bound,
                         density=not self.extended, scale=scale)

    ax.grid(True)

    txt = _param_text(describe(self), arg, error)

    if print_par: print txt

    ax.text(parmloc[0], parmloc[1], txt, ha='left', va='top',
            transform=ax.transAxes)


def draw_compare(f, arg, edges, data, errors=None, normed=False, parts=False):
    """
    this need to be rewritten
    """
    #arg is either map or tuple
    if isinstance(arg, dict): arg = parse_arg(f,arg,1)
    x = (edges[:-1]+edges[1:])/2.0
    bw = np.diff(edges)
    yf = vector_apply(f, x, *arg)
    total = np.sum(data)
    if normed:
        plt.errorbar(x, data/bw/total, errors/bw/total, fmt='.b')
        plt.plot(x,yf,'r',lw=2)
    else:
        plt.errorbar(x,data,errors,fmt='.b')
        plt.plot(x,yf*bw,'r',lw=2)

    #now draw the parts
    if parts:
        if not hasattr(f,'nparts') or not hasattr(f,'eval_parts'):
            warn(RuntimeWarning('parts is set to True but function does '
                            'not have nparts or eval_parts method'))
        else:
            scale = bw if not normed else 1.
            nparts = f.nparts
            parts_val = list()
            for tx in x:
                val = f.eval_parts(tx,*arg)
                parts_val.append(val)
            py = zip(*parts_val)
            for y in py:
                tmpy = np.array(y)
                plt.plot(x, tmpy*scale, lw=2, alpha=0.5)
    plt.grid(True)
    return x,yf,data


def draw_pdf(f, arg, bound, bins=100, scale=1.0, density=True, **kwds):
    edges = np.linspace(bound[0], bound[1], bins)
    return draw_pdf_with_edges(f, arg, edges, scale=scale,
                                density=density, **kwds)


def draw_pdf_with_edges(f, arg, edges, scale=1.0, density=True, **kwds):
    x = (edges[:-1]+edges[1:])/2.0
    bw = np.diff(edges)
    scale *= bw if not density else 1.
    return draw_pdf_with_midpoints(f, arg, x, scale=scale, **kwds)


def draw_pdf_with_midpoints(f, arg, x, scale=1.0, **kwds):
    if isinstance(arg, dict): arg = parse_arg(f, arg, 1)
    yf = vector_apply(f, x, *arg)
    yf *= scale
    plt.plot(x, yf, **kwds)
    return x,yf


#draw comprison between function given args and data
def draw_compare_hist(f, arg, data, bins=100, bound=None,weights=None,
                      normed=False, use_w2=False, parts=False):
    """
    draw histogram of data with poisson error bar and f(x,*arg).

    ::

        np.rannd(10000)
        f = gaussian
        draw_compare_hist(f, {'mean':0,'sigma':1}, data, normed=True)

    **Arguments**

        - **f**
        - **arg** argument pass to f. Can be dictionary or list.
        - **data** data array
        - **bins** number of bins. Default 100.
        - **bound** optional boundary of plot in tuple form. If `None` is
          given, the bound is determined from min and max of the data. Default
          `None`
        - **weights** weights array. Default None.
        - **normed** optional normalized data flag. Default False.
        - **use_w2** scaled error down to the original statistics instead of
          weighted statistics.
        - **parts** draw parts of pdf. (Works with AddPdf and Add2PdfNorm).
          Default False.
    """
    if bound is None: bound = minmax(data)
    h,e = np.histogram(data, bins=bins, range=bound, weights=weights)
    err = None
    if weights is not None and use_w2:
       err,_ = np.histogram(data, bins=bins, range=bound,
                            weights=weights*weights)
       err = np.sqrt(err)
    else:
        err = np.sqrt(h)
    return draw_compare(f, arg, e, h, err, normed, parts)



#cython: embedsignature=True
import numpy as np
from .util import describe

def rename(f, newarg):
    """
    Rename function parameters.

    ::

        def f(x,y,z):
            return do_something(x,y,z)
        g = rename(f, ['x','a','b'])
        print describe(g) # ['x','a','b']

        #g is equivalent to
        def g_equiv(x,a,b):
            return f(x,a,b)

    **Arguments**

        - **f** callable object
        - **newarg** list of new argument names

    **Returns**

        function with new argument.

    """
    return FakeFunc(f, newarg)

def merge_func_code(*arg, prefix=None, skip_first=False):
    """
    merge function arguments.::

        def f(x,y,z): return do_something(x,y,z)
        def g(x,z,p): return do_something(x,y,z)
        fc, pos = merge_func_code(f,g)
        #fc is now ('x','y','z','p')

    **Arguments**

        - **prefix** optional array of prefix string to add to each function
          argument. Default None.::

                def f(x,y,z): return do_something(x,y,z)
                def g(x,z,p): return do_something(x,y,z)
                fc, pos = merge_func_code(f,g,prefix=['f','g'], skip_first=True)
                #fc now ('x','f_y','f_z','f_p')

        - **skip_first** option boolean to skip prefixing the first argument
          of each function. This should normally be true when prefix is not
          None.

    **Return**

        tuple(Merged Func Code, position array)

        position array **p[i][j]** indicates where in the argument list to
        obtain argument **j** for functin **i**

    .. seealso::

        functor.construct_arg
    """
    assert(prefix is None or len(prefix)==len(arg))
    all_arg = []

    for i,f in enumerate(arg):
        tmp = []
        first = skip_first
        for vn in describe(f):
            newv = vn
            if not first and prefix is not None:
                newv = prefix[i]+newv
            first = False
            tmp.append(newv)
        all_arg.append(tmp)

    #now merge it
    #FIXME: do something smarter
    merge_arg = []
    for a in all_arg:
        for v in a:
            if v not in merge_arg:
                merge_arg.append(v)

    #build the map list of numpy int array
    pos = []
    for a in all_arg:
        tmp = []
        for v in a:
            tmp.append(merge_arg.index(v))
        pos.append(np.array(tmp,dtype=np.int))
    return MinimalFuncCode(merge_arg), pos

class MinimalFuncCode:
    def __init__(self, arg):
        self.co_varnames = tuple(arg)
        self.co_argcount = len(arg)


    def append(self, varname):
        tmp = list(self.co_varnames)
        tmp.append(varname)
        self.co_varnames = tuple(tmp)
        self.co_argcount = len(self.co_varnames)


#fake copying func_code with renaming parameters
#and docking off parameters from the front
class FakeFuncCode:
    def __init__(self, f, prmt=None, dock=0, append=None):
        #f can either be tuple or function object
        self.co_varnames=describe(f)
        self.co_argcount=len(self.co_varnames)
        self.co_argcount-=dock
        self.co_varnames = self.co_varnames[dock:]

        if prmt is not None:#rename parameters from the front
            for i,p in enumerate(prmt):
                self.co_varnames[i] = p

        if isinstance(append,str): append = [append]

        if append is not None:
            old_count = self.co_argcount
            self.co_argcount+=len(append)
            self.co_varnames = tuple(
                                    list(self.co_varnames[:old_count])+
                                    append+
                                    list(self.co_varnames[old_count:]))


cdef class FakeFunc:
    cdef f
    cdef public func_code
    cdef public func_defaults
    cdef public __name__

    def __init__(self, f, prmt=None):
        self.f = f
        self.func_code = FakeFuncCode(f,prmt)
        self.func_defaults = getattr(f,'func_defaults',None)
        self.__name__ = getattr(f,'__name__','unnamed')

    def __call__(self, *arg):
        return self.f(*arg)



#cython: embedsignature=True
cimport cython

from libc.math cimport exp, pow, fabs, log, sqrt, sinh, tgamma, log1p, abs
cdef double pi = 3.14159265358979323846264338327
import numpy as np
cimport numpy as np
from util import describe
from funcutil import MinimalFuncCode
np.import_array()

cdef double badvalue = 1e-300
cdef double smallestdiv = 1e-10

cdef class Polynomial:
    cdef int order
    cdef public object func_code
    cdef public object func_defaults
    def __init__(self,order,xname='x'):
        """
        .. math::
            f(x; c_i) = \sum_{i < \\text{order}} c_i x^i

        User can supply order as integer in which case it uses (c_0....c_n+1)
        default or the list of coefficient name which the first one will be the
        lowest order and the last one will be the highest order
        """
        varnames = None
        argcount = 0
        if isinstance(order, int):
            if order<0 : raise ValueError('order must be >=0')
            self.order = order
            varnames = ['c_%d'%i for i in range(order+1)]
        else: #use supply list of coeffnames #to do check if it's list of string
            if len(order)<=0: raise ValueError('need at least one coefficient')
            self.order=len(order)-1 #yep -1 think about it
            varnames = order
        varnames.insert(0,xname) #insert x in front
        self.func_code = MinimalFuncCode(varnames)
        self.func_defaults = None

    def __call__(self,*arg):
        cdef double x = arg[0]
        cdef double t
        cdef double ret=0.
        cdef int iarg
        cdef int numarg = self.order+1 #number of coefficient
        cdef int i
        for i in range(numarg):
            iarg = i+1
            t = arg[iarg]
            if i > 0:
                ret+=pow(x,i)*t
            else:#avoid 0^0
                ret += t
        return ret


cpdef double doublegaussian(double x, double mean, double sigmal, double sigmar):
    """
    Unnormalized double gaussian

    .. math::
        f(x;mean,\sigma_l,\sigma_r) =
        \\begin{cases}
            \exp \left[ -\\frac{1}{2} \left(\\frac{x-mean}{\sigma_l}\\right)^2 \\right], & \mbox{if } x < mean \\\\
            \exp \left[ -\\frac{1}{2} \left( -\\frac{x-mean}{\sigma_r}\\right)^2 \\right], & \mbox{if } x >= mean
        \end{cases}

    """
    cdef double ret = 0.
    cdef double sigma = 0.
    sigma = sigmal if x < mean else sigmar
    if sigma < smallestdiv:
        ret = badvalue
    else:

        d = (x-mean)/sigma
        d2 = d*d
        ret = exp(-0.5*d2)
    return ret

cpdef double ugaussian(double x, double mean, double sigma):
    """
    Unnormalized gaussian

    .. math::
        f(x; mean, \sigma) = \exp \left[ -\\frac{1}{2} \\left( \\frac{x-mean}{\sigma} \\right)^2 \\right]

    """
    cdef double ret = 0
    if sigma < smallestdiv:
        ret = badvalue
    else:
        d = (x-mean)/sigma
        d2 = d*d
        ret = exp(-0.5*d2)
    return ret


cpdef double gaussian(double x, double mean, double sigma):
    """
    Normalized gaussian.

    .. math::
        f(x; mean, \sigma) = \\frac{1}{\sqrt{2\pi}\sigma}
        \exp \left[  -\\frac{1}{2} \left(\\frac{x-mean}{\sigma}\\right)^2 \\right]
    """
    cdef double badvalue = 1e-300
    cdef double ret = 0
    if sigma < smallestdiv:
        ret = badvalue
    else:
        d = (x-mean)/sigma
        d2 = d*d
        ret = 1/(sqrt(2*pi)*sigma)*exp(-0.5*d2)
    return ret


cpdef double crystalball(double x,double alpha,double n,double mean,double sigma):
    """
    Unnormalized crystal ball function

    .. math::
        f(x;\\alpha,n,mean,\sigma) =
        \\begin{cases}
            \exp\left( -\\frac{1}{2} \delta^2 \\right) & \mbox{if } \delta>-\\alpha \\\\
            \left( \\frac{n}{|\\alpha|} \\right)^n \left( \\frac{n}{|\\alpha|} - |\\alpha| - \delta \\right)^{-n} 
            \exp\left( -\\frac{1}{2}\\alpha^2\\right)
            & \mbox{if } \delta \leq \\alpha
        \end{cases}

    where
        - :math:`\delta = \\frac{x-mean}{\sigma}`

    .. note::
        http://en.wikipedia.org/wiki/Crystal_Ball_function
    """
    cdef double d = 0.
    cdef double ret = 0
    cdef double A = 0
    cdef double B = 0
    if sigma < smallestdiv:
        ret = badvalue
    elif fabs(alpha) < smallestdiv:
        ret = badvalue
    elif n<1.:
        ret = badvalue
    else:
        d = (x-mean)/sigma
        if d > -alpha :
            ret = exp(-0.5*d**2)
        else:
            al = fabs(alpha)
            A=pow(n/al,n)*exp(-al**2/2.)
            B=n/al-al
            ret = A*pow(B-d,-n)
    return ret


#Background stuff
cpdef double argus(double x, double c, double chi, double p):
    """
    Unnormalized argus distribution

    .. math::
        f(x;c,\chi,p) = \\frac{x}{c^2} \left( 1-\\frac{x^2}{c^2} \\right)
            \exp \left( - \\frac{1}{2} \chi^2 \left( 1 - \\frac{x^2}{c^2} \\right) \\right)

    .. note::
        http://en.wikipedia.org/wiki/ARGUS_distribution
    """
    if c<smallestdiv:
        return badvalue
    if x>c:
        return 0.
    cdef double xc = x/c
    cdef double xc2 = xc*xc
    cdef double ret = 0

    ret = xc/c*pow(1.-xc2,p)*exp(-0.5*chi*chi*(1-xc2))

    return ret


cpdef double cruijff(double x, double m0, double sigma_L, double sigma_R, double alpha_L, double alpha_R):
    """
    Unnormalized cruijff function

    .. math::
        f(x;m_0, \sigma_L, \sigma_R, \\alpha_L, \\alpha_R) =
        \\begin{cases}
            \exp\left(-\\frac{(x-m_0)^2}{2(\sigma_{L}^2+\\alpha_{L}(x-m_0)^2)}\\right)
            & \mbox{if } x<m_0 \\\\
            \exp\left(-\\frac{(x-m_0)^2}{2(\sigma_{R}^2+\\alpha_{R}(x-m_0)^2)}\\right)
            & \mbox{if } x<m_0 \\\\
        \end{cases}

    """
    cdef double dm2 = (x-m0)*(x-m0)
    cdef double demon=0.
    cdef double ret=0.
    if x<m0:
        denom = 2*sigma_L*sigma_L+alpha_L*dm2
        if denom<smallestdiv:
            return 0.
        return exp(-dm2/denom)
    else:
        denom = 2*sigma_R*sigma_R+alpha_R*dm2
        if denom<smallestdiv:
            return 0.
        return exp(-dm2/denom)


cpdef double linear(double x, double m, double c):
    """
    Linear function.

    .. math::
        f(x;m,c) = mx+c
    """
    cdef double ret = m*x+c
    return ret


cpdef double poly2(double x, double a, double b, double c):
    """
    Parabola

    .. math::
        f(x;a,b,c) = ax^2+bx+c
    """
    cdef double ret = a*x*x+b*x+c
    return ret


cpdef double poly3(double x, double a, double b, double c, double d):
    """
    Polynomial of third order

    .. math::
        f(x; a,b,c,d) =ax^3+bx^2+cx+d

    """
    cdef double x2 = x*x
    cdef double x3 = x2*x
    cdef double ret = a*x3+b*x2+c*x+d
    return ret


cpdef double novosibirsk(double x, double width, double peak, double tail):
    """
    Unnormalized Novosibirsk

    .. math::
        f(x;\sigma, x_0, \Lambda) = \exp\left[
            -\\frac{1}{2} \\frac{\left( \ln q_y \\right)^2 }{\Lambda^2} + \Lambda^2 \\right] \\\\
        q_y(x;\sigma,x_0,\Lambda) = 1 + \\frac{\Lambda(x-x_0)}{\sigma} \\times
        \\frac{\sinh \left( \Lambda \sqrt{\ln 4} \\right)}{\Lambda \sqrt{\ln 4}}

    where
        - width = :math:`\sigma`
        - peak = :math:`m_0`
        - tail = :math:`\Lambda`
    """
    #credit roofit implementation
    cdef double qa
    cdef double qb
    cdef double qc
    cdef double qx
    cdef double qy
    cdef double xpw
    cdef double lqyt
    if width < smallestdiv: return badvalue
    xpw = (x-peak)/width
    if fabs(tail) < 1e-7:
        qc = 0.5*xpw*xpw
    else:
        qa = tail*sqrt(log(4.))
        if qa < smallestdiv: return badvalue
        qb = sinh(qa)/qa
        qx = xpw*qb
        qy = 1.+tail*qx
        if qy > 1e-7:
            lqyt = log(qy)/tail
            qc =0.5*lqyt*lqyt + tail*tail
        else:
            qc=15.
    return exp(-qc)


cpdef double rtv_breitwigner(double x, double m, double gamma):
    """
    Normalized Relativistic Breit-Wigner

    .. math::
        f(x; m, \Gamma) = N\\times \\frac{1}{(x^2-m^2)^2+m^2\Gamma^2}

    where

    .. math::
        N = \\frac{2 \sqrt{2} m \Gamma  \gamma }{\pi \sqrt{m^2+\gamma}}

    and

    .. math::
        \gamma=\sqrt{m^2\left(m^2+\Gamma^2\\right)}


    .. seealso::
        :func:`cauchy`

    """
    cdef double mm = m*m
    cdef double xm = x*x-mm
    cdef double gg = gamma*gamma
    cdef double s = sqrt(mm*(mm+gg))
    cdef double N = (2*sqrt(2)/pi)*m*gamma*s/sqrt(mm+s)
    return N/(xm*xm+mm*gg)


cpdef double cauchy(double x, double m, double gamma):
    """
    Cauchy distribution aka non-relativistic Breit-Wigner

    .. math::
        f(x, m, \gamma) = \\frac{1}{\pi \gamma \left[ 1+\left( \\frac{x-m}{\gamma} \\right)^2\\right]}

    .. seealso::
        :func:`breitwigner`
    """
    cdef double xmg = (x-m)/gamma
    return 1/(pi*gamma*(1+xmg*xmg))



from distutils.core import setup, Extension
import distutils.util
import subprocess
import numpy as np
# source_suffix = 'pyx'
# has_cython = False
# try:
#     from Cython.Distutils import build_ext
#     has_cython = True
# except:
#     pass

#cmdclass={}
#if not has_cython:
source_suffix = 'c'
#else:
#    cmdclass = {'build_ext':build_ext}


costfunc = Extension('probfit.costfunc',
                    sources = ['probfit/costfunc.'+source_suffix],
                    include_dirs= [np.get_include()],
                    extra_link_args = [])

pdf = Extension('probfit.pdf',
        sources = ['probfit/pdf.'+source_suffix],
        include_dirs= [np.get_include()],
        extra_link_args = [])

libstat = Extension('probfit._libstat',
        sources = ['probfit/_libstat.'+source_suffix],
        include_dirs= [np.get_include()],
        extra_link_args = [])

funcutil = Extension('probfit.funcutil',
        sources = ['probfit/funcutil.'+source_suffix],
        include_dirs= [np.get_include()],
        extra_link_args = [])

functor = Extension('probfit.functor',
        sources = ['probfit/functor.'+source_suffix],
        include_dirs= [np.get_include()],
        extra_link_args = [])

execfile('probfit/info.py')

setup (
        #cmdclass=cmdclass,
        name = 'probfit',
        version = __version__,
        description = 'Distribution Fitting/Regression Library',
        long_description=''.join(open('README.rst').readlines()[4:]),
        author='Piti Ongmongkolkul',
        author_email='piti118@gmail.com',
        url='https://github.com/piti118/dist_fit',
        package_dir = {'probfit': 'probfit'},
        packages = ['probfit'],
        ext_modules = [costfunc, pdf, libstat, funcutil, functor],
        requires=['numpy (>=1.6)','iminuit (>=1.0.2)'],
        classifiers=[
            "Programming Language :: Python",
            'Topic :: Scientific/Engineering :: Physics',
            'Topic :: Scientific/Engineering :: Mathematics',
            'Intended Audience :: Science/Research',
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: MIT License'
        ],
)



cpdef double doublegaussian(double x, double mean, double sigmal, double sigmar)

cpdef double ugaussian(double x, double mean, double sigma)

cpdef double gaussian(double x, double mean, double sigma)

cpdef double crystalball(double x,double alpha,double n,double mean,double sigma)

cpdef double argus(double x, double c, double chi, double p)

cpdef double cruijff(double x, double m0, double sigma_L, double sigma_R, double alpha_L, double alpha_R)

cpdef double linear(double x, double m, double c)

cpdef double poly2(double x, double a, double b, double c)

cpdef double poly3(double x, double a, double b, double c, double d)

cpdef double novosibirsk(double x, double width, double peak, double tail)

cpdef double rtv_breitwigner(double x, double m, double gamma)

cpdef double cauchy(double x, double m, double gamma)



#cython: embedsignature=True
import numpy as np
cimport numpy as np
from libc.math cimport exp,pow,fabs,log,tgamma,lgamma,log1p,sqrt
from warnings import warn
from probfit_warnings import LogWarning
np.import_array()

cpdef np.ndarray[np.double_t] _vector_apply(f,np.ndarray[np.double_t] x,tuple arg):
    cdef int i
    cdef int n = len(x)
    cdef np.ndarray[np.double_t] ret = np.empty(n,dtype=np.double)#fast_empty(n)
    cdef double tmp
    for i in range(n):
        tmp = f(x[i],*arg)
        ret[i]=tmp
    return ret


cpdef double csum(np.ndarray x):
    cdef int i
    cdef np.ndarray[np.double_t] xd = x
    cdef int n = len(x)
    cdef double s=0.
    for i in range(n):
        s+=xd[i]
    return s


cpdef double integrate1d_with_edges(f,np.ndarray edges, double bw, tuple arg) except *:
    cdef np.ndarray[np.double_t] y = _vector_apply(f,edges,arg)
    return csum(y*bw)-0.5*(y[0]+y[-1])*bw#trapezoid


#to do runge kutta or something smarter
cpdef double integrate1d(f, tuple bound, int nint, tuple arg=None) except*:
    if arg is None: arg = tuple()
    cdef double ret = 0
    cdef np.ndarray[np.double_t] edges = np.linspace(bound[0],bound[1],nint)
    #cdef np.ndarray[np.double_t] bw = edges[1:]-edges[:-1]
    cdef double bw = edges[1]-edges[0]
    return integrate1d_with_edges(f,edges,bw,arg)


#compute x*log(y/x) to a good precision especially when y~x
cpdef double xlogyx(double x,double y):
    cdef double ret
    if x<1e-100:
        warn(LogWarning('x is really small return 0'))
        return 0.
    if x<y:
        ret = x*log1p((y-x)/x)
    else:
        ret = -x*log1p((x-y)/y)
    return ret


#compute w*log(y/x) where w < x and goes to zero faster than x
cpdef double wlogyx(double w,double y, double x):
    if x<1e-100:
        warn(LogWarning('x is really small return 0'))
        return 0.
    if x<y:
        ret = w*log1p((y-x)/x)
    else:
        ret = -w*log1p((x-y)/y)
    return ret

#these are performance critical code
cpdef double compute_bin_lh_f(f,
                    np.ndarray[np.double_t] edges,
                    np.ndarray[np.double_t] h, #histogram,
                    np.ndarray[np.double_t] w2,
                    double N, #sum of h
                    tuple arg, double badvalue,
                    bint extend, bint use_sumw2) except *:
    """
    Calculate binned likelihood. The behavior depends whether extended
    likelihood and whether use_sumw2 is requested

    - ``extended=False`` and ``use_sumw2=False``. The second term in the sum is
       to subtract off the minimum to get better precision.

      .. math::
            $-\sum_i h_i * log(f(x_i,...)*N*bin_width) - (h_i-f(x_i,...)*N*bin_width)$

    - ``extended=False`` and ``use_sumw2=True``

    - ``extended=True`` and ``use_sumw2=False``

    - ``extended=True`` and ``use_sumw2=True``
    """
    cdef int i
    cdef int n = len(edges)

    cdef np.ndarray[np.double_t] fedges = _vector_apply(f,edges,arg)
    cdef np.ndarray[np.double_t] midvalues = (fedges[1:]+fedges[:-1])/2
    cdef double ret = 0.
    cdef double bw = 0.
    #TODO: change this. This is super inefficient.
    #cdef double E = integrate1d(f,(edges[0],edges[-1]),10000,arg)
    cdef double factor=0.
    cdef double th=0.
    cdef double tw=0.
    cdef double tm=0.
    for i in range(n-1):#h has length of n-1
        #ret -= h[i]*log(midvalues[i])#non zero subtraction
        bw = edges[i+1]-edges[i]
        th = h[i]
        tm = midvalues[i]
        if not extend:
            if not use_sumw2:
                ret -= xlogyx(th,tm*N*bw)+(th-tm*bw*N)
                #h[i]*log(midvalues[i]/nh[i]) #subtracting h[i]*log(h[i]/(N*bw))
                #the second term is added for added precision near the minimum
            else:
                if w2[i]<1e-200: continue
                tw = w2[i]
                #tw = sqrt(tw)
                factor = th/tw
                ret -= factor*(wlogyx(th,tm*N*bw,th)+(th-tm*bw*N))
        else:
            #print 'h',h[i],'midvalues',midvalues[i]*bw
            if not use_sumw2:
                ret -= xlogyx(th,tm*bw)+(th-tm*bw)
            else:
                if w2[i]<1e-200: continue
                tw = w2[i]
                #tw = sqrt(tw)
                factor = th/tw
                ret -= factor*(wlogyx(th,tm*bw,th)+(th-tm*bw))

    return ret


cpdef double compute_nll(f,np.ndarray data,w,arg,double badvalue) except *:
    cdef int i=0
    cdef double lh=0
    cdef double nll=0
    cdef double ret=0
    cdef double thisdata=0
    cdef np.ndarray[np.double_t] data_ = data
    cdef int data_len = len(data)
    cdef np.ndarray[np.double_t] w_
    if w is None:
        for i in range(data_len):
            thisdata = data_[i]
            lh = f(thisdata,*arg)
            if lh<=0:
                ret = badvalue
                break
            else:
                ret+=log(lh)
    else:
        w_ = w
        for i in range(data_len):
            thisdata = data_[i]
            lh = f(thisdata,*arg)
            if lh<=0:
                ret = badvalue
                break
            else:
                ret+=log(lh)*w_[i]
    return -1*ret


cpdef double compute_chi2_f(f,
                    np.ndarray[np.double_t] x,
                    np.ndarray[np.double_t] y,
                    np.ndarray[np.double_t] error,
                    np.ndarray[np.double_t] weights,
                    tuple arg) except *:
    cdef int usew = 1 if weights is not None else 0
    cdef int usee = 1 if error is not None else 0
    cdef int i
    cdef int datalen = len(x)
    cdef double diff = 0.
    cdef double fx = 0.
    cdef double ret = 0.
    cdef double err = 0.
    for i in range(datalen):
        fx = f(x[i],*arg)
        diff = fx-y[i]
        if usee==1:
            err = error[i]
            if err<1e-10:
                raise ValueError('error contains value too small or negative')
            diff = diff/error[i]
        diff *= diff
        if usew==1:
            diff*=weights[i]
        ret += diff
    return ret


cpdef double compute_bin_chi2_f(f,
                np.ndarray[np.double_t] x, np.ndarray[np.double_t] y,
                np.ndarray[np.double_t]error, np.ndarray[np.double_t] binwidth,
                np.ndarray[np.double_t]weights, tuple arg) except *:
    cdef int usew = 1 if weights is not None else 0
    cdef int usee = 1 if error is not None else 0
    cdef int i
    cdef int datalen = len(x)
    cdef double diff
    cdef double fx
    cdef double ret = 0.
    cdef double err
    cdef double bw
    for i in range(datalen):
        bw = binwidth[i]
        fx = f(x[i],*arg)*bw
        diff = fx-y[i]
        if usee==1:
            err = error[i]
            if err<1e-10:
                raise ValueError('error contains value too small or negative')
            diff = diff/error[i]
        diff *= diff
        if usew==1:
            diff*=weights[i]
        ret += diff
    return ret


cpdef double compute_chi2(np.ndarray[np.double_t] actual,
                        np.ndarray[np.double_t] expected,
                        np.ndarray[np.double_t] err) except *:
    cdef int i=0
    cdef int maxi = len(actual)
    cdef double a
    cdef double e
    cdef double er
    cdef double ret = 0.
    for i in range(maxi):
        e = expected[i]
        a = actual[i]
        er = err[i]
        if er<1e-10:
            raise ValueError('error contains value too small or negative')
        ea = (e-a)/er
        ea *= ea
        ret += ea
    return ret

cpdef compute_cdf(np.ndarray[np.double_t] pdf, np.ndarray[np.double_t] x):
    cdef int i
    cdef int n = len(pdf)
    cdef double lpdf
    cdef double rpdf
    cdef bw
    cdef np.ndarray[np.double_t] ret
    ret = np.zeros(n)
    ret[0] = 0
    for i in range(1,n):#do a trapezoid sum
        lpdf = pdf[i]
        rpdf = pdf[i-1]
        bw = x[i]-x[i-1]
        ret[i] = 0.5*(lpdf+rpdf)*bw + ret[i-1]
    return ret

#invert cdf useful for making toys
cpdef invert_cdf(np.ndarray[np.double_t] r,
                np.ndarray[np.double_t] cdf,
                np.ndarray[np.double_t] x):
    cdef np.ndarray[np.int_t] loc = np.searchsorted(cdf,r,'right')
    cdef int n = len(r)
    cdef np.ndarray[np.double_t] ret = np.zeros(n)
    cdef int i = 0
    cdef int ind
    cdef double ly
    cdef double lx
    cdef double ry
    cdef double rx
    cdef double minv
    for i in range(n):
        ind = loc[i]
        #print ind,i,len(loc),len(x)
        ly = cdf[ind-1]
        ry = cdf[ind]
        lx = x[ind-1]
        rx = x[ind]
        minv = (rx-lx)/(ry-ly)
        ret[i] = minv*(r[i]-ly)+lx
    return ret



import unittest
from probfit.util import describe, parse_arg

class Func_Code:
    def __init__(self,varname):
        self.co_varnames=varname
        self.co_argcount=len(varname)

class Func1:
    def __init__(self):
        pass
    def __call__(self,x,y):
        return (x-2.)**2 + (y-5.)**2 + 10

class Func2:
    def __init__(self):
        self.func_code = Func_Code(['x','y'])
    def __call__(self,*arg):
        return (arg[0]-2.)**2 + (arg[1]-5.)**2 + 10

def func3(x,y):
    return 0.2*(x-2.)**2 + (y-5.)**2 + 10


def func4(x,y,z):
    return 0.2*(x-2.)**2 + 0.1*(y-5.)**2 + 0.25*(z-7.)**2 + 10

class TestUtil(unittest.TestCase):
    def setUp(self):
        self.f1=Func1()
        self.f2=Func2()
        self.f3=func3

    def iterable_equal(self, x, y):
        self.assertEqual(list(x),list(y))

    def test_parse_arg(self):
        td = {'x':1,'y':2}
        ts = parse_arg(self.f1,td)
        self.assertEqual(ts,(1,2))

    def test_describe(self):
        self.iterable_equal(describe(self.f1),('x','y'))
        self.iterable_equal(describe(self.f2),['x','y'])
        self.iterable_equal(describe(self.f3),('x','y'))
    


__all__ = [
    'AddPdfNorm',
    'AddPdf',
    'BinnedChi2',
    'BinnedLH',
    'Chi2Regression',
    'Convolve',
    'Extended',
    'Normalized',
    'Polynomial',
    'UnbinnedLH',
    'argus',
    'cruijff',
    'cauchy',
    'rtv_breitwigner',
    'crystalball',
    'describe',
    'doublegaussian',
    'draw_compare',
    'draw_compare_hist',
    'draw_pdf',
    'draw_pdf_with_edges',
    'extended',
    'fit_binlh',
    'fit_binx2',
    'fit_uml',
    'fwhm_f',
    'gaussian',
    'gen_toy',
    'gen_toyn',
    'linear',
    'merge_func_code',
    'normalized',
    'novosibirsk',
    'poly2',
    'poly3',
    'rename',
    'SimultaneousFit',
    'try_binlh',
    'try_chi2',
    'try_uml',
    'ugaussian',
    '__version__'
    ]

from .costfunc import UnbinnedLH, BinnedLH, Chi2Regression, BinnedChi2,\
                      SimultaneousFit
from .pdf import doublegaussian, ugaussian, gaussian, crystalball,\
                 argus, cruijff, linear, poly2, poly3, novosibirsk,\
                 Polynomial, cauchy, rtv_breitwigner
from .toy import gen_toy, gen_toyn
from .util import *
from .oneshot import *
from .statutil import *
from .plotting import *
from .funcutil import *
from .decorator import *

from .functor import Normalized, Extended, Convolve, AddPdf, AddPdfNorm
from .info import __version__



from .costfunc import UnbinnedLH, BinnedChi2, BinnedLH
from ._libstat import _vector_apply
from iminuit import Minuit
from matplotlib import pyplot as plt
import numpy as np
import numpy.random as npr
import itertools as itt
import collections
from warnings import warn
from .util import parse_arg, describe
from .nputil import minmax


def fit_uml(f, data, quiet=False, print_level=0, *arg, **kwd):
    """
    perform unbinned likelihood fit
    :param f: pdf
    :param data: data
    :param quiet: if not quite draw latest fit on fail fit
    :param printlevel: minuit printlevel
    :return:
    """
    uml = UnbinnedLH(f, data)
    m = Minuit(uml, print_level=print_level, **kwd)
    m.set_strategy(2)
    m.set_up(0.5)
    m.migrad()
    if not m.migrad_ok() or not m.matrix_accurate():
        if not quiet:
            plt.figure()
            uml.show()
            print m.values
    return (uml, m)


def fit_binx2(f, data, bins=30, bound=None, print_level=0, quiet=False, *arg, **kwd):
    """
    perform chi^2 fit
    :param f:
    :param data:
    :param bins:
    :param range:
    :param printlevel:
    :param quiet:
    :param arg:
    :param kwd:
    :return:
    """
    uml = BinnedChi2(f, data, bins=bins, bound=bound)
    m = Minuit(uml, print_level=print_level, **kwd)
    m.set_strategy(2)
    m.set_up(1)
    m.migrad()
    if not m.migrad_ok() or not m.matrix_accurate():
        if not quiet:
            plt.figure()
            uml.show()
            print m.values

    return (uml, m)


def fit_binlh(f, data, bins=30,
              bound=None, quiet=False, weights=None, use_w2=False,
              print_level=0, pedantic=True, extended=False,
              *arg, **kwd):
    """
    perform bin likelihood fit
    :param f:
    :param data:
    :param bins:
    :param range:
    :param quiet:
    :param weights:
    :param use_w2:
    :param printlevel:
    :param pedantic:
    :param extended:
    :param arg:
    :param kwd:
    :return:
    """
    uml = BinnedLH(f, data, bins=bins, bound=bound,
                    weights=weights, use_w2=use_w2, extended=extended)
    m = Minuit(uml, print_level=print_level, pedantic=pedantic, **kwd)
    m.set_strategy(2)
    m.set_up(0.5)
    m.migrad()
    if not m.migrad_ok() or not m.matrix_accurate():
        if not quiet:
            plt.figure()
            uml.show()
            print m.values
        #m.minos()
    return (uml, m)


def tuplize(x):
    """
    :param x:
    :return:
    """
    if  isinstance(x, collections.Iterable):
        return x
    else:
        return tuple([x])


def pprint_arg(vnames, value):
    """
    pretty print argument
    :param vnames:
    :param value:
    :return:
    """
    ret = ''
    for name, v in zip(vnames, value):
        ret += '%s=%s;' % (name, str(v))
    return ret;


def try_uml(f, data, bins=40, fbins=1000, *arg, **kwd):
    fom = UnbinnedLH(f, data)
    narg = f.func_code.co_argcount
    vnames = f.func_code.co_varnames[1:narg]
    my_arg = [tuplize(kwd[name]) for name in vnames]
    h, e, _ = plt.hist(data, bins=bins, normed=True, histtype='step')
    vx = np.linspace(e[0], e[-1], fbins)
    first = True
    minfom = 0
    minarg = None
    for thisarg in itt.product(*my_arg):
        vy = _vector_apply(f, vx, thisarg)
        plt.plot(vx, vy, '-', label=pprint_arg(vnames, thisarg))
        thisfom = fom(*thisarg)
        if first or thisfom < minfom:
            minfom = thisfom
            minarg = thisarg
            first = False
    leg = plt.legend(fancybox=True)
    leg.get_frame().set_alpha(0.5)
    ret = dict((k,v) for k, v in zip(vnames, minarg))
    return ret

def try_binlh(f, data, weights=None, bins=40, fbins=1000, show='both', extended=False,
        bound=None, *arg, **kwd):
    if bound is None: bound = minmax(data)
    fom = BinnedLH(f, data,extended=extended,bound=bound)
    narg = f.func_code.co_argcount
    vnames = f.func_code.co_varnames[1:narg]
    my_arg = [tuplize(kwd[name]) for name in vnames]
    h, e = None, None
    if show == 'both':
        h, e, _ = plt.hist(data, bins=bins, range=bound, histtype='step', weights=weights, normed=not extended)
    else:
        h, e = np.histogram(data, bins=bins, range=bound, weights=weights, normed=not extended)
    bw = e[1] - e[0]

    vx = np.linspace(e[0], e[-1], fbins)
    first = True
    minfom = 0
    minarg = None
    for thisarg in itt.product(*my_arg):
        vy = _vector_apply(f, vx, thisarg)
        if extended: vy*=bw
        plt.plot(vx, vy, '-', label=pprint_arg(vnames, thisarg))
        thisfom = fom(*thisarg)
        if first or thisfom < minfom:
            minfom = thisfom
            minarg = thisarg
            first = False
    leg = plt.legend(fancybox=True)
    leg.get_frame().set_alpha(0.5)
    ret = dict((k,v) for k, v in zip(vnames, minarg))
    return ret


def try_chi2(f, data, weights=None, bins=40, fbins=1000, show='both', *arg, **kwd):
    fom = BinnedChi2(f, data)
    narg = f.func_code.co_argcount
    vnames = f.func_code.co_varnames[1:narg]
    my_arg = [tuplize(kwd[name]) for name in vnames]
    h, e = None, None
    if show == 'both':
        h, e, _ = plt.hist(data, bins=bins, histtype='step', weights=weights)
    else:
        h, e = np.histogram(data, bins=bins, weights=weights)
    bw = e[1] - e[0]
    vx = np.linspace(e[0], e[-1], fbins)
    first = True
    minfom = 0
    minarg = None
    for thisarg in itt.product(*my_arg):
        vy = _vector_apply(f, vx, thisarg) * bw
        plt.plot(vx, vy, '-', label=pprint_arg(vnames, thisarg))
        thisfom = fom(*thisarg)
        if first or thisfom < minfom:
            minfom = thisfom
            minarg = thisarg
            first = False
    leg = plt.legend(fancybox=True)
    leg.get_frame().set_alpha(0.5)
    ret = dict((k, v) for k, v in zip(vnames, minarg))
    return ret


# def randfr(r):
#     """
#     generate a uniform random number with in range r
#     :param r: tuple range
#     :return: float
#     """
#     b = r[1]
#     a = r[0]
#     return np.random.ranf() * (b - a) + a

# def guess_initial(alg, f, data, ntry=100, guessrange=(-100, 100), draw=False, *arg, **kwd):
#     """
#     This is very bad at the moment don't use it
#     """
#     fom = alg(f, data, *arg, **kwd)
#     first = True
#     minfom = 0
#     minparam = ()
#     narg = fom.func_code.co_argcount
#     vnames = fom.func_code.co_varnames[:narg]
#     ranges = {}
#     for vname in vnames:
#         if 'limit_' + vname in kwd:
#             ranges[vname] = kwd['limit_' + vname]
#         else:
#             ranges[vname] = guessrange

#     for i in xrange(ntry):
#         arg = []
#         for vname in vnames:
#             arg.append(randfr(ranges[vname]))
#         try:
#             thisfom = fom(*arg)
#             if first or thisfom < minfom:
#                 first = False
#                 minfom = thisfom
#                 minparam = arg
#         except ZeroDivisionError:
#             pass

#     print minparam, minfom
#     ret = {}
#     for vname, bestguess in zip(vnames, minparam):
#         ret[vname] = bestguess
#     if draw:
#         fom(*minparam)
#         fom.draw()
#     return ret
